
#ifndef _MC_GRID_3D_SCALAR_HPP_
#define _MC_GRID_3D_SCALAR_HPP_

#include <MC_grid_3d.hpp>
#include <MC_int_vector.hpp>
#include <MC_v3d.hpp>

namespace mesh_conv
{
    namespace implicit
    {
        class MC_potential_sphere;
    }

    /** \brief 3D grid for scalar density (use float to save space) */
    class MC_grid_3d_scalar
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        MC_grid_3d_scalar();
        /** \brief direct constructor */
        MC_grid_3d_scalar(const MC_int_vector& given_size,const MC_v3d& real_size=MC_v3d(1,1,1),const MC_v3d& offset=MC_v3d(0,0,0));


        // ********************************************* //
        // ********************************************* //
        //  Size
        // ********************************************* //
        // ********************************************* //

        /** \brief get size */
        const MC_int_vector& size() const;
        /** \brief check if a position is within the bounds (real coordinates) */
        bool is_within_bound(const MC_v3d& x) const;
        /** \brief check if a position is within the bounds (local voxel coordinates) */
        bool is_within_bound_voxel(const MC_int_vector& x) const;


        // ********************************************* //
        // ********************************************* //
        //  Fill
        // ********************************************* //
        // ********************************************* //

        /** \brief fill with a constant value */
        void fill(const float& value);

        // ********************************************* //
        // ********************************************* //
        //  Dimension and conversion
        // ********************************************* //
        // ********************************************* //

        /** \brief get real size */
        const MC_v3d& dimension() const;
        /** \brief get real size */
        MC_v3d& dimension();

        /** \brief get offset */
        const MC_v3d& offset() const;
        /** \brief get offset */
        MC_v3d& offset();

        /** \brief convert from real coordinates to voxel index */
        MC_v3d convert_real_to_voxel_index(const MC_v3d& x) const;
        /** \brief convert from voxel index to real coordinates */
        MC_v3d convert_voxel_index_to_real(const MC_v3d& u) const;


        // ********************************************* //
        // ********************************************* //
        //  get and set values
        // ********************************************* //
        // ********************************************* //

        /** \brief get value from a given voxel index */
        const float& get(const MC_int_vector& u) const;
        /** \brief get value from a given voxel index */
        float& get(const MC_int_vector& u);
        /** \brief set value from a given voxel index */
        void set(const MC_int_vector& u,const float& value);


        /** \brief get value using linear interpolation */
        float operator()(const MC_v3d& x) const;


        // ********************************************* //
        // ********************************************* //
        //  Add potential object
        // ********************************************* //
        // ********************************************* //

        /** \brief add an implicit sphere into the grid by sampling values */
        void add_potential(const implicit::MC_potential_sphere& sphere_implicit);


        // ********************************************* //
        // ********************************************* //
        //  Evaluator
        // ********************************************* //
        // ********************************************* //

        /** \brief compute linear interpolation */
        float linear_interpolation(const MC_v3d& x) const;



    private:

        /** \brief internal grid */
        MC_grid_3d<float> potential;

        /** \brief dimension conversion from voxel index to real value */
        MC_v3d dimension_internal;
        /** \brief offset conversion from voxel index to real value */
        MC_v3d offset_internal;

    };
}

#endif
