
#ifndef _MC_GRID_SPATIAL_HASHING_HPP_
#define _MC_GRID_SPATIAL_HASHING_HPP_

#include <MC_grid_3d.hpp>

#include <set>
#include <algorithm>

namespace mesh_conv
{
    /** \brief regular grid to hash index position in space */
    class MC_grid_index_spatial_hashing
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        MC_grid_index_spatial_hashing();
        /** \brief build grid of a given size */
        MC_grid_index_spatial_hashing(const MC_int_vector& grid_size);

        // ********************************************* //
        // ********************************************* //
        //  Size
        // ********************************************* //
        // ********************************************* //

        /** \brief get the size of the grid */
        MC_int_vector size() const;

        /** \brief check if the current position is within bounds */
        bool is_in_bound(const MC_int_vector& position) const;

        // ********************************************* //
        // ********************************************* //
        //  Global filling
        // ********************************************* //
        // ********************************************* //

        /** \brief empty every case */
        void clear();


        // ********************************************* //
        // ********************************************* //
        //  Index
        // ********************************************* //
        // ********************************************* //

        /** \brief add the given index to a position of the grid */
        void add_index(const MC_int_vector& X,const int index);
        /** \brief delete the given index to a position of the grid */
        void delete_index(const MC_int_vector& X,const int index);

        /** \brief get the set of index of a given position */
        std::set<int>& get(const MC_int_vector& X);
        /** \brief get the set of index of a given position */
        const std::set<int>& get(const MC_int_vector& X) const;

        /** \brief get all the index within a given sphere */
        MC_int_vector_vector get_voxel_within_sphere(const MC_v3d& center,const double& radius) const;


        // ********************************************* //
        // ********************************************* //
        //  Neighbooring map
        // ********************************************* //
        // ********************************************* //

        /** add a sphere in the hash map */
        void add_sphere(const MC_v3d& center,const double& radius,const int& id);
        /** get the index which are without the circumference */
        std::set<int> neighboors_index_in_radius(const MC_v3d& center,const double& radius) const;

    private:

        /** \brief the grid of index */
        MC_grid_3d<std::set<int> > index_grid;

    };

}


#endif
