
#ifndef _MC_PARTICLE_ENGINE_HPP_
#define _MC_PARTICLE_ENGINE_HPP_

#include <MC_sphere_physic.hpp>
#include <MC_grid_index_spatial_hashing.hpp>

#include <vector>
#include <map>
#include <algorithm>

namespace mesh_conv
{

    /** \brief Particle engine based on spheres */
    class MC_particle_engine
    {
    public:

        // ********************************************* //
        // ********************************************* //
        // CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** brief init an empty particle engine */
        MC_particle_engine();

        // ********************************************* //
        // ********************************************* //
        // Particles handler
        // ********************************************* //
        // ********************************************* //

        /** \brief add a particle */
        void add(const MC_sphere_physic& new_particle);

        /** \brief get the particle system */
        const std::map<unsigned int,MC_sphere_physic>& particle_system() const;


        // ********************************************* //
        // ********************************************* //
        // Spatial grid
        // ********************************************* //
        // ********************************************* //

        /** \brief get the spatial hash grid */
        MC_grid_index_spatial_hashing& grid_spatial_hash();
        /** \brief get the spatial hash grid */
        const MC_grid_index_spatial_hashing& grid_spatial_hash() const;


        // ********************************************* //
        // ********************************************* //
        // Evolve in time
        // ********************************************* //
        // ********************************************* //

        /** \brief evolve time for every particles */
        void evolve(const bool& is_collision_detection=true,const bool& is_potential=false);

    private:

        /** \brief the vector of particles */
        std::map<unsigned int,MC_sphere_physic> particles;

        /** \brief the current time step */
        double time_step;

        /** \brief the spatial hash table for intersection speed-up */
        MC_grid_index_spatial_hashing spatial_hash;
        /** \brief the spatial hash table for proximity speed-up */
        MC_grid_index_spatial_hashing spatial_hash_proximity;

        /** \brief follow the particle id max */
        static unsigned int id_max;

        /** \brief dimension of the hash table */
        unsigned int dim_spatial_hash;
        /** \brief dimension of the proximity hash table */
        unsigned int dim_spatial_hash_proximity;



    };

}

#endif 
