
#ifndef _MC_SPHERE_GEOMETRY_HPP_
#define _MC_SPHERE_GEOMETRY_HPP_

#include <MC_v3d.hpp>

namespace mesh_conv
{
    /** \brief Model for a geometric sphere */
    class MC_sphere_geometry
    {
    public:

        // ********************************************* //
        // ********************************************* //
        // CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        MC_sphere_geometry();
        /** \brief direct constructor */
        MC_sphere_geometry(const MC_v3d& _center,const double& _radius);
        /** \brief copy constuctor */
        MC_sphere_geometry(const MC_sphere_geometry& sphere);

        // ********************************************* //
        // ********************************************* //
        // Get parameters
        // ********************************************* //
        // ********************************************* //

        /** \brief get the center of the sphere */
        MC_v3d& center();
        /** \brief get the center of the sphere */
        const MC_v3d& center() const;

        /** \brief get the radius of the sphere */
        double& radius();
        /** \brief get the radius of the sphere */
        const double& radius() const;

        // ********************************************* //
        // ********************************************* //
        // Collision detection
        // ********************************************* //
        // ********************************************* //

        /** \brief check if the current sphere collide with an other one */
        bool is_collide(const MC_sphere_geometry& sphere);


    private:

        /** \brief the center of the sphere */
        MC_v3d center_internal;
        /** \brief the radius of the sphere */
        double radius_internal;

    };
}

#endif
