
#ifndef _MC_SPHERE_PHYSIC_HPP_
#define _MC_SPHERE_PHYSIC_HPP_

#include <MC_sphere_geometry.hpp>

namespace mesh_conv
{
    /** \brief Physical particle model associated to a sphere */
    class MC_sphere_physic : public MC_sphere_geometry
    {
    public:

        // ********************************************* //
        // ********************************************* //
        // CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        MC_sphere_physic();
        /** \brief direct constructor */
        MC_sphere_physic(const MC_v3d& position,const double& radius);
        /** \brief direct constructor */
        MC_sphere_physic(const MC_v3d& position,const double& radius,const MC_v3d& speed_init);


        // ********************************************* //
        // ********************************************* //
        // Get parameters
        // ********************************************* //
        // ********************************************* //

        /** \brief get speed */
        MC_v3d& speed();
        /** \brief get speed */
        const MC_v3d& speed() const;

        /** \brief get force */
        MC_v3d& force();
        /** \brief get force */
        const MC_v3d& force() const;

        /** \brief get mass */
        double& mass();
        /** \brief get mass */
        const double& mass() const;

        /** \brief get damping factor */
        double& damping_factor();
        /** \brief get damping factor */
        const double& damping_factor() const;

        // ********************************************* //
        // ********************************************* //
        // Time evolution
        // ********************************************* //
        // ********************************************* //

        /** \brief evolve through time */
        void evolve(const double& delta_t);


        // ********************************************* //
        // ********************************************* //
        // Constraints
        // ********************************************* //
        // ********************************************* //


        /** \brief ensure the particle stays in a unit cube and modify speed accordingly */
        void constraint_in_unit_cube();

        /** \brief ensure two particles does not collide and modify speed accordingly */
        static void constraint_no_collision_sphere(MC_sphere_physic* sphere_1,MC_sphere_physic* sphere_2);

    private:


        /** \brief current speed of the particle */
        MC_v3d speed_internal;
        /** \brief extrernal forces */
        MC_v3d force_internal;

        /** \brief particle mass */
        double mass_internal;
        /** \brief damping factor */
        double damping_factor_internal;


    };
}

#endif 
