
#include <MC_grid_3d_scalar_slicer.hpp>

#include <MC_grid_3d_scalar.hpp>
#include <MC_mesh_index_vector.hpp>

#include <cmath>

namespace mesh_conv
{
    MC_double_vector MC_grid_3d_scalar_slicer::slice(const MC_grid_3d_scalar& potential,const MC_mesh_index_vector& mesh)
    {
        MC_double_vector v=MC_double_vector::zeros(mesh.vertex_number());
        for(unsigned int k=0,N=mesh.vertex_number();k<N;++k)
            v[k]=potential(mesh.point_set()(k));
        return v;
    }

    void MC_grid_3d_scalar_slicer::export_square_ppm_file(const std::string& filename,const MC_double_vector& value,const unsigned int& N0,const unsigned int& N1)
    {
        if(N0*N1 != value.size())
        {exception::MC_exception("Error in MC_grid_3d_scalar_slicer::export_square_ppm_file size does not match ");}

        std::ofstream stream(filename.c_str());
        if(stream.good()!=true)
        {exception::MC_exception("Cannot open file "+filename+" in MC_grid_3d_scalar_slicer::export_square_ppm_file");}
        export_square_ppm(stream,value,N0,N1);
        stream.close();
    }

    void MC_grid_3d_scalar_slicer::export_square_ppm(std::ofstream& stream,const MC_double_vector& value,const unsigned int& N0,const unsigned int& N1)
    {
        if(N0*N1 != value.size())
        {exception::MC_exception("Error in MC_grid_3d_scalar_slicer::export_square_ppm size does not match ");}

        stream<<"P3"<<std::endl; // magic number
        stream<<N0<<" "<<N1<<std::endl;
        stream<<"255"<<std::endl;
        for(unsigned int k0=0;k0<N0;k0++)
        {
            for(unsigned int k1=0;k1<N1;k1++)
            {
                int v = static_cast<int>(std::min(std::max(static_cast<double>(value[k0+k1*N0]),static_cast<double>(0)),static_cast<double>(255)));
                stream<<v<<" "<<v<<" "<<v<<std::endl;
            }
        }
    }

}
