

#include <MC_sphere_physic.hpp>

namespace mesh_conv
{
    MC_sphere_physic::MC_sphere_physic()
            :MC_sphere_geometry(),speed_internal(MC_v3d(0,0,0)),force_internal(MC_v3d(0,0,0)),mass_internal(1),damping_factor_internal(0.01)
    {}
    MC_sphere_physic::MC_sphere_physic(const MC_v3d& position,const double& radius)
            :MC_sphere_geometry(position,radius),speed_internal(MC_v3d(0,0,0)),force_internal(MC_v3d(0,0,0)),mass_internal(1),damping_factor_internal(0.01)
    {}
    MC_sphere_physic::MC_sphere_physic(const MC_v3d& position,const double& radius,const MC_v3d& speed_init)
            :MC_sphere_geometry(position,radius),speed_internal(speed_init),force_internal(MC_v3d(0,0,0)),mass_internal(1),damping_factor_internal(0.01)
    {}


    MC_v3d& MC_sphere_physic::speed()
    {return speed_internal;}
    const MC_v3d& MC_sphere_physic::speed() const
    {return speed_internal;}
    MC_v3d& MC_sphere_physic::force()
    {return force_internal;}
    const MC_v3d& MC_sphere_physic::force() const
    {return force_internal;}

    double& MC_sphere_physic::mass(){return mass_internal;}
    const double& MC_sphere_physic::mass() const{return mass_internal;}

    double& MC_sphere_physic::damping_factor(){return damping_factor_internal;}
    const double& MC_sphere_physic::damping_factor() const{return damping_factor_internal;}

    void MC_sphere_physic::evolve(const double& delta_t)
    {

        MC_v3d acceleration = force_internal/mass_internal;

        //explicit integation
        speed_internal = (1-damping_factor_internal)*speed_internal + delta_t*acceleration;
        center() += delta_t*speed_internal;

        //force_internal.set_zero();
    }

    void MC_sphere_physic::constraint_in_unit_cube()
    {

        MC_v3d& c=center();
        MC_v3d& s=speed();

        const double& r=radius();

        // ********************** //

        // TO DO :

        // si la sphere sort de [0,1] alors on change la vitesse s

        // ********************** //


    }

    void MC_sphere_physic::constraint_no_collision_sphere(MC_sphere_physic* sphere_1,MC_sphere_physic* sphere_2)
    {

        if(! (sphere_1->is_collide(*sphere_2)) )
            return;

        const MC_v3d& c1=sphere_1->center();
        const MC_v3d& c2=sphere_2->center();

        const double& r1=sphere_1->radius();
        const double& r2=sphere_2->radius();

        double L=(c1-c2).norm();
        MC_v3d u=(c1-c2)/L;

        double r_total=(r1+r2);
        double L_penetration=r_total-L;

        double epsilon=1e-5;

        //project on the contact surface
        sphere_1->center() += +(L_penetration+epsilon)/2.0 * u;
        sphere_2->center() += -(L_penetration+epsilon)/2.0 * u;



        // ********************** //
        // TO DO :

        // changer la vitesse
        MC_v3d nouvelle_vitesse_1=sphere_1->speed();
        MC_v3d nouvelle_vitesse_2=sphere_1->speed();
        // ********************** //


        sphere_1->speed()=nouvelle_vitesse_1;
        sphere_2->speed()=nouvelle_vitesse_2;

    }

}
