/*
**    Mesh Converter
**    Copyright (C) 2009 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MC_HELPER_STL_HPP_
#define _MC_HELPER_STL_HPP_

#include <iostream>
#include <vector>
#include <map>

namespace mesh_conv
{
    /** \brief Helper class to deal with stl container */
    class MC_helper_stl
    {
    public:

        /** \brief convert the argument2 of a map to a vector */
        template <class TA,class TB>
                static std::vector<TB> map_arg2_to_vector(const std::map<TA,TB>& m)
        {
            typename std::map<TA,TB>::const_iterator it=m.begin();
            typename std::map<TA,TB>::const_iterator it_end=m.end();

            std::vector <TB> vec;
            for(;it!=it_end;++it)
                vec.push_back(it->second);
            return vec;
        }

        /** \brief reverse a map from map<A,B> going to map<B,A> */
        template <class TA,class TB>
                static std::map<TB,TA> reverse_map(const std::map<TA,TB>& m)
        {
            std::map <TB,TA> reversed;
            typename std::map <TA,TB> :: const_iterator it=m.begin();
            typename std::map <TA,TB> :: const_iterator it_end=m.end();
            for(;it!=it_end;++it)
                reversed.insert(std::make_pair(it->second,it->first));
            return reversed;
        }

    private:
    };
}

#endif
