/*
**    Mesh Converter
**    Copyright (C) 2009  Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MC_INT_PAIR_HPP_
#define _MC_INT_PAIR_HPP_

#include <stdlib.h>
#include <iostream>




namespace mesh_conv
{

    /** \brief Container class for pair of int

     (u0,u1) == (u1,u0) */
    class MC_int_pair
    {
    public:


        //************************************************//
        //************************************************//
        //CONSTRUCTORS
        //************************************************//
        //************************************************//

        /** \brief Empty constructor */
        MC_int_pair();
        /** \brief copy constructor */
        MC_int_pair(const MC_int_pair& i);
        /** \brief direct constructor */
        MC_int_pair(const int& _u0,const int& _u1);
        /** \brief constructor from std::pair(a,b) */
        template <typename U,typename V>
                MC_int_pair(const std::pair<U,V>& p):u0(p.first),u1(p.second){}
        /** \brief destructor */
        ~MC_int_pair();

        //************************************************//
        //************************************************//
        //GET
        //************************************************//
        //************************************************//


        /** \brief  access (k must be 0 or 1) */
        const int& operator[](const int& k) const;
        /** \brief access (k must be 0 or 1) */
        int& operator[](const int& k);
        /** \brief access (k must be 0 or 1) */
        const int& operator()(const int& k) const;
        /** \brief access (k must be 0 or 1) */
        int& operator()(const int& k);


        //************************************************//
        //************************************************//
        //Math Operator
        //************************************************//
        //************************************************//

        /** \brief test equality */
        friend bool operator==(const MC_int_pair& p1,const MC_int_pair& p2);
        /** \brief test equality */
        friend bool operator!=(const MC_int_pair& p1,const MC_int_pair& p2);

        /** \brief affectation */
        MC_int_pair& operator=(const MC_int_pair& to_copy);


        /** \brief add a value to the pair
            \return a pair (x0,x1)+a=(x0+a,x1+a)
          */
        friend MC_int_pair operator+(const MC_int_pair& p,const int& a);
        /** \brief substract a value to the pair
            \return a pair (x0,x1)-a=(x0-a,x1-a)
          */
        friend MC_int_pair operator-(const MC_int_pair& p,const int& a);



        /** \brief ordonate the int_pair in the same order (lower value is the first one) */
        MC_int_pair ordonated() const;

        /** \brief assert the index is either 0 or 1
            \return nothing, but stop the programm if it goes outside bounds
        */
        void assert_bounds(const int& u) const;

        //************************************************//
        //************************************************//
        //Input Output
        //************************************************//
        //************************************************//

        /** output */
        friend std::ostream& operator << (std::ostream& flux,const MC_int_pair& _v);


    private:

        /// the pair
        int u0,u1;

    };


    /** \brief a class to ordonnate the MC_int_pair */
    class MC_int_pair_less
    {
    public:
        bool operator()(const MC_int_pair& a,const MC_int_pair& b) const
        {
            MC_int_pair ca=a.ordonated();
            MC_int_pair cb=b.ordonated();

            if( (ca[0]<cb[0]) || (ca[0]==cb[0] && ca[1]<cb[1]) )
                return true;
            return false;
        }
    private:
    };

}
#endif
