/*
**    Mesh Converter
**    Copyright (C) 2009  Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _MC_INT_PAIR_UNIQUE_HPP_
#define _MC_INT_PAIR_UNIQUE_HPP_

#include <iostream>


namespace mesh_conv
{
    class MC_int_vector;
    class MC_int_pair;

    class MC_int_pair_unique
    {
    public:


        //************************************************//
        //************************************************//
        //CONSTRUCTORS
        //************************************************//
        //************************************************//

        /** \brief Classical constructor with zero defaut value*/
        MC_int_pair_unique();
        /** \brief copy constructor */
        MC_int_pair_unique(const MC_int_pair_unique& I);
        /** \brief constructor from int_pair */
        MC_int_pair_unique(const MC_int_pair& p);
        /** \brief direct constructor */
        MC_int_pair_unique(const int& _u0,const int& _u1);
        /** \brief direct constructor with a vector */
        MC_int_pair_unique(const MC_int_vector& u);
        /** \brief Classical destroy */
        ~MC_int_pair_unique();

        //************************************************//
        //************************************************//
        //Information
        //************************************************//
        //************************************************//


        /** \brief assert the index is either 0 or 1
            \return nothing, but stop the programm if it goes outside bounds
        */
        void assert_bounds(const int& u) const;


        //************************************************//
        //************************************************//
        //Get
        //************************************************//
        //************************************************//

        /** \brief test equality */
        friend bool operator==(const MC_int_pair_unique& p1,const MC_int_pair_unique& p2);
        /** \brief test equality */
        friend bool operator!=(const MC_int_pair_unique& p1,const MC_int_pair_unique& p2);

        /** \brief access (k must be 0 or 1) */
        const int& operator[](const int& k) const;
        /** \brief access (k must be 0 or 1) */
        int& operator[](const int& k);
        /** \brief access (k must be 0 or 1) */
        const int& operator()(const int& k) const;
        /** \brief access (k must be 0 or 1) */
        int& operator()(const int& k);

        //************************************************//
        //************************************************//
        //INPUT/OUTPUT
        //************************************************//
        //************************************************//

        /** output */
        friend std::ostream& operator << (std::ostream& stream,const MC_int_pair_unique& _v);


 private:

        /** the private pair */
        int u0,u1;

    };


    /** \brief a comparator functor */
    class MC_int_pair_unique_less
    {
    public:
        bool operator ()(const MC_int_pair_unique& p0,const MC_int_pair_unique& p1) const
        {

            if( p0(0)<p1(0) || (p0(0)==p1(0) && p0(1)<p1(1)) )
                return true;

            return false;
        }
    };


}

#endif
