
/*
**    Mesh Converter
**    Copyright (C) 2009  Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _MC_IO_OBJ_HPP_
#define _MC_IO_OBJ_HPP_


#include <MC_v3d_vector.hpp>
#include <MC_connectivity_index.hpp>

#include <iostream>
#include <fstream>


namespace mesh_conv
{
    class MC_mesh_index_vector;
    class MC_mesh_index_vector_draw;

     /** \brief Helper class to read and write mesh in a obj format */
    class MC_io_obj
    {
        public:

        /** \brief read an input stream in a obj format
            \return <point_set,connectivity> */
        static std::pair <MC_v3d_vector,MC_connectivity_index> read_obj(std::istream& stream);

        /** \brief read an input stream in a obj format for the texture
            \return <texture_vector,texture_connectivity> */
        static std::pair <MC_v3d_vector,MC_connectivity_index> read_obj_texture(std::istream& stream);
        /** \brief read an input file in a obj format for the texture
            \return <texture_vector,texture_connectivity> */
        static std::pair <MC_v3d_vector,MC_connectivity_index> read_obj_texture_file(const std::string& file);

        /** \brief read an input stream in a obj format for the normal
            \return <normal_vector,normal_connectivity> */
        static std::pair <MC_v3d_vector,MC_connectivity_index> read_obj_normal(std::istream& stream);

        /** \brief read an input file in a obj format
            \return <point_set,connectivity> */
        static std::pair <MC_v3d_vector,MC_connectivity_index> read_obj_file(const std::string& filename);


        /** \brief write input stream in obj format */
        static std::ostream& write_obj(std::ostream& stream,const MC_mesh_index_vector& to_write);
        /** \brief write input stream in obj format */
        static std::ostream& write_obj(std::ostream& stream,const MC_mesh_index_vector_draw& to_write);

        /** \brief write input stream in obj format */
        template <typename T>
                static const MC_mesh_index_vector& write_obj_file(const std::string& string_file,const T& to_write)
        {
            std::ofstream ofile(string_file.c_str());
            if(ofile.good()!=true)
            {std::cout<<"Error in MC_io_obj::write_obj_file("<<string_file<<",MC_mesh_index_vector), cannot open the file"<<std::endl;exit(-1);}

            MC_io_obj::write_obj(ofile,to_write);
            ofile.close();
            return to_write;
        }


        private:
    };
}

#endif
