
#include <MC_int_pair.hpp>

namespace mesh_conv
{


    MC_int_pair::MC_int_pair(){u0=0;u1=0;}
    MC_int_pair::MC_int_pair(const MC_int_pair& input){u0=input.u0;u1=input.u1;}
    MC_int_pair::MC_int_pair(const int& _u0,const int& _u1){u0=_u0;u1=_u1;}
    MC_int_pair::~MC_int_pair(){}

    bool operator==(const MC_int_pair& p0,const MC_int_pair& p1)
    {
        if( ( (p0.u0==p1.u0) && (p0.u1==p1.u1) ) || ( (p0.u0==p1.u1) && (p0.u1==p1.u0) ))
            return true;
        return false;
    }

    bool operator!=(const MC_int_pair& p0,const MC_int_pair& p1)
    {
        if( ( (p0.u0==p1.u0) && (p0.u1==p1.u1) ) || ( (p0.u0==p1.u1) && (p0.u1==p1.u0) ))
            return false;
        return true;
    }

    MC_int_pair& MC_int_pair::operator=(const MC_int_pair& to_copy)
                                       {
        u0=to_copy[0];
        u1=to_copy[1];
        return *this;
    }

    std::ostream& operator<<(std::ostream& stream, const MC_int_pair& p)
    {
        stream<<"("<<p[0]<<","<<p[1]<<")";
        return stream;
    }

    void MC_int_pair::assert_bounds(const int& u) const
    {
        if(u!=0 && u!=1)
        {std::cout<<"Error in MC_int_pair::assert_bounds("<<u<<"), index must be 0 or 1"<<std::endl;exit(-1);}
    }

    const int& MC_int_pair::operator[](const int& k) const
    {
        assert_bounds(k);
        if(k==0) return u0;
        return u1;
    }
    int& MC_int_pair::operator[](const int& k)
    {
        assert_bounds(k);
        if(k==0) return u0;
        return u1;
    }
    const int& MC_int_pair::operator()(const int& k) const
    {
        assert_bounds(k);
        if(k==0) return u0;
        return u1;
    }
    int& MC_int_pair::operator()(const int& k)
    {
        assert_bounds(k);
        if(k==0) return u0;
        return u1;
    }

    MC_int_pair MC_int_pair::ordonated() const
    {
        if(u0<u1)
            return MC_int_pair(u0,u1);
        return MC_int_pair(u1,u0);
    }


    MC_int_pair operator+(const MC_int_pair& p,const int& a)
    {return MC_int_pair(p[0]+a,p[1]+a);}
    MC_int_pair operator-(const MC_int_pair& p,const int& a)
    {return MC_int_pair(p[0]-a,p[1]-a);}

}
