
#include <MC_mesh_index_vector.hpp>

#include <MC_segment.hpp>
#include <MC_polygon.hpp>
#include <MC_triangle.hpp>
#include <MC_matrix.hpp>
#include <MC_int_vector.hpp>
#include <MC_int_vector_vector.hpp>
#include <MC_int_pair.hpp>
#include <MC_double_vector_vector.hpp>
#include <MC_io_off.hpp>
#include <MC_helper_stl.hpp>
#include <MC_string_helper.hpp>

#include <MC_io_off.hpp>
#include <MC_io_obj.hpp>
#include <MC_int_pair.hpp>

#include <sstream>


#include <map>
#include <set>
#include <vector>





namespace mesh_conv
{


    MC_mesh_index_vector::MC_mesh_index_vector(){}

    int MC_mesh_index_vector::polygon_number() const {return connectivity_mesh.size();}
    int MC_mesh_index_vector::vertex_number() const {return point_set_mesh.size();}


    MC_mesh_index_vector& MC_mesh_index_vector::add_vertex(const MC_v3d& v) {point_set_mesh.add(v); return *this;}
    MC_mesh_index_vector& MC_mesh_index_vector::add_connectivity_index(const MC_int_vector& c) {connectivity_mesh.add(c); return *this;}


    MC_connectivity_index& MC_mesh_index_vector::connectivity() {return connectivity_mesh;}
    const MC_connectivity_index& MC_mesh_index_vector::connectivity() const {return connectivity_mesh;}
    MC_v3d_vector& MC_mesh_index_vector::point_set() {return point_set_mesh;}
    const MC_v3d_vector& MC_mesh_index_vector::point_set() const {return point_set_mesh;}

    int MC_mesh_index_vector::polygon_size(const int& k_polygon) const
    {
        //check size
        if( k_polygon<0 || k_polygon>=connectivity_mesh.size() )
        {std::cout<<"Error in MC_mesh_index_vector::polygon_size("<<k_polygon<<"), with connectivity size="<<connectivity_mesh.size()<<std::endl;exit(-1);}

        return connectivity_mesh[k_polygon].size();
    }
    MC_polygon MC_mesh_index_vector::get_polygon(const int& k_index) const
    {
        //check size
        if( k_index<0 || k_index>=connectivity_mesh.size() )
        {std::cout<<"Error in MC_mesh_index_vector::polygon_size("<<k_index<<"), with connectivity size="<<connectivity_mesh.size()<<std::endl;exit(-1);}
        return point_set_mesh(connectivity_mesh[k_index]);
    }

    MC_mesh_index_vector MC_mesh_index_vector::build_cube()
    {
        MC_v3d_vector points = MC_v3d_vector(MC_v3d(0,0,0))<<
                               MC_v3d(1,0,0)<<
                               MC_v3d(1,1,0)<<
                               MC_v3d(0,1,0)<<
                               MC_v3d(1,0,1)<<
                               MC_v3d(1,1,1)<<
                               MC_v3d(0,1,1)<<
                               MC_v3d(0,0,1);

        MC_connectivity_index conn = MC_connectivity_index(MC_int_vector(0,1,2,3))<<
                                     MC_int_vector(1,4,5,2)<<
                                     MC_int_vector(2,5,6,3)<<
                                     MC_int_vector(3,6,7,0)<<
                                     MC_int_vector(0,7,4,1)<<
                                     MC_int_vector(6,5,4,7);

        //std::cout<<conn<<std::endl;

        return MC_mesh_index_vector(points,conn);
    }

    MC_mesh_index_vector::MC_mesh_index_vector(const MC_v3d_vector& _point_set,const MC_connectivity_index& _connectivity)
    {point_set_mesh=_point_set;connectivity_mesh=_connectivity;}
    MC_mesh_index_vector::MC_mesh_index_vector(const std::pair<MC_v3d_vector,MC_connectivity_index>& _mesh)
    {*this=MC_mesh_index_vector(_mesh.first,_mesh.second);}

    MC_mesh_index_vector operator+(const MC_mesh_index_vector& vec,const MC_v3d& to_add)
    {return MC_mesh_index_vector(vec.point_set()+to_add,vec.connectivity());}
    MC_mesh_index_vector operator-(const MC_mesh_index_vector& vec,const MC_v3d& to_sub)
    {return MC_mesh_index_vector(vec.point_set()-to_sub,vec.connectivity());}
    MC_mesh_index_vector operator+(const MC_mesh_index_vector& vec,const MC_v3d_vector& to_add)
    {return MC_mesh_index_vector(vec.point_set()+to_add,vec.connectivity());}
    MC_mesh_index_vector operator-(const MC_mesh_index_vector& vec,const MC_v3d_vector& to_sub)
    {return MC_mesh_index_vector(vec.point_set()-to_sub,vec.connectivity());}
    MC_mesh_index_vector operator*(const MC_mesh_index_vector& vec,const double& to_mult)
    {return MC_mesh_index_vector(vec.point_set()*to_mult,vec.connectivity());}
    MC_mesh_index_vector operator*(const double& to_mult,const MC_mesh_index_vector& vec)
    {return vec*to_mult;}
    MC_mesh_index_vector operator/(const MC_mesh_index_vector& vec,const double& to_subdiv)
    {return MC_mesh_index_vector(vec.point_set()/to_subdiv,vec.connectivity());}

    MC_mesh_index_vector& MC_mesh_index_vector::operator+=(const MC_v3d& to_add)
    {point_set_mesh+=to_add;return *this;}
    MC_mesh_index_vector& MC_mesh_index_vector::operator-=(const MC_v3d& to_sub)
    {point_set_mesh-=to_sub;return *this;}
    MC_mesh_index_vector& MC_mesh_index_vector::operator+=(const MC_v3d_vector& to_add)
    {point_set_mesh+=to_add;return *this;}
    MC_mesh_index_vector& MC_mesh_index_vector::operator-=(const MC_v3d_vector& to_sub)
    {point_set_mesh-=to_sub;return *this;}
    MC_mesh_index_vector& MC_mesh_index_vector::operator*=(const double& to_mult)
    {point_set_mesh*=to_mult;return *this;}
    MC_mesh_index_vector& MC_mesh_index_vector::operator/=(const double& to_subdiv)
    {point_set_mesh/=to_subdiv;return *this;}

    MC_mesh_index_vector operator*(const MC_matrix& M,const MC_mesh_index_vector& mesh)
    {return MC_mesh_index_vector(M*mesh.point_set(),mesh.connectivity());}
    MC_mesh_index_vector& MC_mesh_index_vector::operator*=(const MC_matrix& M)
    {point_set_mesh*=M;return *this;}

    MC_v3d_vector MC_mesh_index_vector::normal_vertex() const
    {


        MC_v3d_vector v_n(point_set().size());
        if(static_cast<int>(connectivity().star().size())!=point_set().size())
        {std::cout<<"Error in MC_v3d_vector MC_mesh_index_vector::normal_vertex(), one star has size "<<connectivity_mesh.star().size()<<", with "<<point_set_mesh.size()<<" vertices, probably need to compute one star"<<std::endl;exit(-1);}
        std::map<int,std::set<int> > star = connectivity().star();

        std::map<int,std::set<int> > :: const_iterator it=star.begin();
        std::map<int,std::set<int> > :: const_iterator it_end=star.end();
        for(;it!=it_end;++it)
        {

            int current_vertex=it->first;
            std::set <int> :: const_iterator it_poly=it->second.begin();
            std::set <int> :: const_iterator it_poly_end=it->second.end();
            for(;it_poly!=it_poly_end;++it_poly)
            {
                int current_poly=*it_poly;
                MC_v3d n=MC_polygon(point_set()(connectivity()(current_poly))).normal();
                v_n[current_vertex]+=n;
            }
        }

        v_n=v_n.normalized();
        return v_n;
    }


    MC_v3d_vector MC_mesh_index_vector::normal_vertex(const MC_v3d_vector& point_set_mesh,const MC_connectivity_index& connectivity_mesh)
    {

        MC_v3d_vector v_n(point_set_mesh.size());
        if(static_cast<int>(connectivity_mesh.star().size())!=point_set_mesh.size())
        {std::cout<<"Error in MC_v3d_vector MC_mesh_index_vector::normal_vertex(MC_v3d_vector,MC_connectivity_index), one star has size "<<connectivity_mesh.star().size()<<", with "<<point_set_mesh.size()<<" vertices, probably need to compute one star"<<std::endl;exit(-1);}
        std::map<int,std::set<int> > star = connectivity_mesh.star();

        std::map<int,std::set<int> > :: const_iterator it=star.begin();
        std::map<int,std::set<int> > :: const_iterator it_end=star.end();
        for(;it!=it_end;++it)
        {

            int current_vertex=it->first;
            std::set <int> :: const_iterator it_poly=it->second.begin();
            std::set <int> :: const_iterator it_poly_end=it->second.end();
            for(;it_poly!=it_poly_end;++it_poly)
            {
                int current_poly=*it_poly;
                MC_v3d n=MC_polygon(point_set_mesh(connectivity_mesh(current_poly))).normal();
                v_n[current_vertex]+=n;
            }
        }

//        for(int k=0;k<v_n.size();++k)
//            if(v_n[k].norm()<0.0001)
//                std::cout<<k<<", "<<v_n[k]<<std::endl;

        v_n=v_n.normalized();

        return v_n;
    }
    MC_double_vector MC_mesh_index_vector::normal_vertex(const MC_double_vector& point_set_mesh,const MC_connectivity_index& connectivity_mesh)
    {
        MC_double_vector v_n=MC_double_vector::zeros(point_set_mesh.size());
        if(static_cast<int>(connectivity_mesh.star().size())!=point_set_mesh.size()/3)
        {std::cout<<"Error in MC_v3d_vector MC_mesh_index_vector::normal_vertex(MC_v3d_vector,MC_connectivity_index), one star has size "<<connectivity_mesh.star().size()<<", with "<<point_set_mesh.size()<<" vertices, probably need to compute one star"<<std::endl;exit(-1);}
        std::map<int,std::set<int> > star = connectivity_mesh.star();

        std::map<int,std::set<int> > :: const_iterator it=star.begin();
        std::map<int,std::set<int> > :: const_iterator it_end=star.end();
        for(;it!=it_end;++it)
        {


            int current_vertex=it->first;
            std::set <int> :: const_iterator it_poly=it->second.begin();
            std::set <int> :: const_iterator it_poly_end=it->second.end();
            for(;it_poly!=it_poly_end;++it_poly)
            {
                int current_poly=*it_poly;
                MC_int_vector index=connectivity_mesh(current_poly);int N_index=index.size();
                MC_polygon p;
                for(int k=0;k<N_index;++k)
                    p.add(MC_v3d(point_set_mesh(3*index(k)+0),point_set_mesh(3*index(k)+1),point_set_mesh(3*index(k)+2)));
                MC_v3d n=p.normal();

                for(int k_dim=0;k_dim<3;++k_dim)
                    v_n[3*current_vertex+k_dim]+=n[k_dim];
            }
        }

        //int N_v=point_set_mesh.size()/3;
        double epsilon=0.00000001;
        for(int k=0;k<point_set_mesh.size()/3;++k)
        {
            MC_v3d temp(v_n(3*k+0),v_n(3*k+1),v_n(3*k+2));
            double nn=temp.norm();
            if(nn>epsilon)
            {v_n(3*k+0)/=nn;v_n(3*k+1)/=nn;v_n(3*k+2)/=nn;}
            else
            {v_n(3*k+0)=1;v_n(3*k+1)=0;v_n(3*k+2)=0;}
        }
        return v_n;
    }

    MC_v3d_vector MC_mesh_index_vector::normal_polygon() const
    {
        int N_poly=polygon_number();
        MC_v3d_vector n=MC_v3d_vector::zeros(N_poly);
        for(int k=0;k<N_poly;++k)
            n[k]=get_polygon(k).normal();
        return n;
    }


    MC_mesh_index_vector MC_mesh_index_vector::build_icosahedron()
    {
        double p = 0.5*(1+powf(5,0.5));
        double f = 1.0/powf(float(1.0+p*p),0.5f);

        //vertices
        MC_v3d_vector icosahedron_vertex=MC_v3d_vector(MC_v3d( p, 1, 0))
                                         <<MC_v3d(-p, 1, 0)
                                         <<MC_v3d( p,-1, 0)
                                         <<MC_v3d(-p,-1, 0)
                                         <<MC_v3d( 1, 0, p)
                                         <<MC_v3d( 1, 0,-p)
                                         <<MC_v3d(-1, 0, p)
                                         <<MC_v3d(-1, 0,-p)
                                         <<MC_v3d( 0, p, 1)
                                         <<MC_v3d( 0,-p, 1)
                                         <<MC_v3d( 0, p,-1)
                                         <<MC_v3d( 0,-p,-1) ;


        icosahedron_vertex*=f;


        //triangulation
        MC_connectivity_index icosahedron_tri=MC_connectivity_index(MC_int_vector(0, 8, 4))
                                              <<MC_int_vector( 0, 5,10)
                                              <<MC_int_vector( 2, 4, 9)
                                              <<MC_int_vector( 2,11, 5)
                                              <<MC_int_vector( 1, 6, 8)
                                              <<MC_int_vector( 1,10, 7)
                                              <<MC_int_vector( 3, 9, 6)
                                              <<MC_int_vector( 3, 7,11)
                                              <<MC_int_vector( 0,10, 8)
                                              <<MC_int_vector( 1, 8,10)
                                              <<MC_int_vector( 2, 9,11)
                                              <<MC_int_vector( 3,11, 9)
                                              <<MC_int_vector( 4, 2, 0)
                                              <<MC_int_vector( 5, 0, 2)
                                              <<MC_int_vector( 6, 1, 3)
                                              <<MC_int_vector( 7, 3, 1)
                                              <<MC_int_vector( 8, 6, 4)
                                              <<MC_int_vector( 9, 4, 6)
                                              <<MC_int_vector(10, 5, 7)
                                              <<MC_int_vector(11, 7, 5) ;


        return MC_mesh_index_vector(icosahedron_vertex,icosahedron_tri);
    }

    MC_mesh_index_vector MC_mesh_index_vector::subdivide_mid_edge_unchanged_boundary(const std::set<MC_int_pair,MC_int_pair_less>& allowed_edge) const
    {
        std::set<MC_int_pair,MC_int_pair_less>::const_iterator allowed_edge_end=allowed_edge.end();

        MC_mesh_index_vector new_mesh;
        new_mesh.point_set()=point_set();


        std::set <int> s=connectivity_mesh.boundary_polygon();
        std::set <MC_int_pair,MC_int_pair_less> bnd=connectivity_mesh.boundary();
        std::set <int> bnd_poly=connectivity_mesh.boundary_polygon();

        std::map <MC_int_pair,int,MC_int_pair_less> newly_added_vertices;

        MC_v3d_vector vertex=point_set_mesh;
        int N_polygon=polygon_number();
        for(int k=0;k<N_polygon;++k)
        {
            MC_int_vector poly=connectivity_mesh(k);
            std::pair<std::vector <MC_polygon>,std::pair<MC_mesh_index_vector,std::pair<MC_int_vector,MC_int_vector> > > current_poly;
            if(bnd_poly.find(k)==bnd_poly.end())//classical subdivision
                current_poly=get_polygon(k).subdivide_mid_edge();
            else
            {
                MC_int_vector constraint_edge;
                int poly_size=poly.size();
                for(int k_poly=0;k_poly<poly_size;++k_poly)
                {
                    MC_int_pair current_edge(poly[k_poly],poly[(k_poly+1)%poly_size]);
                    if(bnd.find(current_edge)!=bnd.end())
                    {
                        std::set<MC_int_pair,MC_int_pair_less>::const_iterator temp_it=allowed_edge.find(current_edge);
                        if(temp_it==allowed_edge_end)
                            constraint_edge.add(k_poly);
                    }
                }
                current_poly=get_polygon(k).subdivide_mid_edge(constraint_edge);
            }



            // goes back to global mesh
            MC_mesh_index_vector local_mesh=current_poly.second.first;
            MC_int_vector extra_vertices=current_poly.second.second.first;
            MC_int_vector edge_of_extra_vertices=current_poly.second.second.second;




            std::map <int,int> extra_local_to_global;
            for(int k_edge=0;k_edge<edge_of_extra_vertices.size();++k_edge)
            {
                MC_int_pair index_edge_local(edge_of_extra_vertices[k_edge],(edge_of_extra_vertices[k_edge]+1)%poly.size());
                MC_int_pair index_edge_global(poly(index_edge_local[0]),poly(index_edge_local[1]));

                std::pair <std::map<MC_int_pair,int,MC_int_pair_less>::iterator,bool> insert_it=newly_added_vertices.insert(std::make_pair(index_edge_global,new_mesh.point_set().size()));
                if(insert_it.second==true)
                {
                    extra_local_to_global[extra_vertices[k_edge]]=new_mesh.point_set().size();
                    new_mesh.point_set().add(local_mesh.point_set()[extra_vertices[k_edge]]);
                }
                else
                    extra_local_to_global[extra_vertices[k_edge]]=insert_it.first->second;
            }

            for(int k_new_poly=0;k_new_poly<local_mesh.polygon_number();++k_new_poly)
            {
                MC_int_vector local_poly=local_mesh.connectivity()(k_new_poly);
                MC_int_vector global_poly;
                for(int k_new_poly_vertex=0;k_new_poly_vertex<local_poly.size();++k_new_poly_vertex)
                {
                    int u=local_poly[k_new_poly_vertex];
                    if(u<poly.size())//old vertices
                        global_poly.add(poly[u]);
                    else
                        global_poly.add(extra_local_to_global[u]);

                }
                new_mesh.connectivity().add(global_poly);
            }
        }
        return new_mesh;
    }



    MC_mesh_index_vector MC_mesh_index_vector::subdivide_barycenter_mid_edge_unchanged_boundary(const std::set<MC_int_pair,MC_int_pair_less>& allowed_edge) const
    {
        MC_mesh_index_vector new_mesh;
        new_mesh.point_set()=point_set();


        std::set <int> s=connectivity_mesh.boundary_polygon();
        std::set <MC_int_pair,MC_int_pair_less> bnd=connectivity_mesh.boundary();
        std::set <int> bnd_poly=connectivity_mesh.boundary_polygon();

        std::map <MC_int_pair,int,MC_int_pair_less> newly_added_vertices;

        MC_v3d_vector vertex=point_set_mesh;
        int N_polygon=polygon_number();
        for(int k=0;k<N_polygon;++k)
        {
            MC_int_vector poly=connectivity_mesh(k);
            std::pair<std::vector <MC_polygon>,std::pair<MC_mesh_index_vector,std::pair<MC_int_vector,MC_int_vector> > > current_poly;
            if(bnd_poly.find(k)==bnd_poly.end())//classical subdivision
                current_poly=get_polygon(k).subdivide_barycenter_mid_edge();
            else
            {
                MC_int_vector constraint_edge;
                int poly_size=poly.size();
                for(int k_poly=0;k_poly<poly_size;++k_poly)
                {
                    MC_int_pair current_edge(poly[k_poly],poly[(k_poly+1)%poly_size]);
                    if(bnd.find(current_edge)!=bnd.end())
                        if(allowed_edge.find(current_edge)==allowed_edge.end())
                            constraint_edge.add(k_poly);
                }
                current_poly=get_polygon(k).subdivide_barycenter_mid_edge(constraint_edge);
            }



            // goes back to global mesh
            MC_mesh_index_vector local_mesh=current_poly.second.first;
            MC_int_vector extra_vertices=current_poly.second.second.first;
            MC_int_vector edge_of_extra_vertices=current_poly.second.second.second;





            std::map <int,int> extra_local_to_global;

            //add the barycenter
            extra_local_to_global[poly.size()]=new_mesh.point_set().size();
            new_mesh.point_set().add(local_mesh.point_set()[poly.size()]);

            for(int k_edge=0;k_edge<edge_of_extra_vertices.size();++k_edge)
            {
                MC_int_pair index_edge_local(edge_of_extra_vertices[k_edge],(edge_of_extra_vertices[k_edge]+1)%poly.size());
                MC_int_pair index_edge_global(poly(index_edge_local[0]),poly(index_edge_local[1]));


                std::pair <std::map<MC_int_pair,int,MC_int_pair_less>::iterator,bool> insert_it=newly_added_vertices.insert(std::make_pair(index_edge_global,new_mesh.point_set().size()));
                if(insert_it.second==true)
                {
                    extra_local_to_global[extra_vertices[k_edge]]=new_mesh.point_set().size();
                    new_mesh.point_set().add(local_mesh.point_set()[extra_vertices[k_edge]]);
                }
                else
                    extra_local_to_global[extra_vertices[k_edge]]=insert_it.first->second;
            }



            for(int k_new_poly=0;k_new_poly<local_mesh.polygon_number();++k_new_poly)
            {
                MC_int_vector local_poly=local_mesh.connectivity()(k_new_poly);
                MC_int_vector global_poly;
                for(int k_new_poly_vertex=0;k_new_poly_vertex<local_poly.size();++k_new_poly_vertex)
                {
                    int u=local_poly[k_new_poly_vertex];
                    if(u<poly.size())//old vertices
                        global_poly.add(poly[u]);
                    else
                        global_poly.add(extra_local_to_global[u]);

                }
                new_mesh.connectivity().add(global_poly);
            }
        }
        return new_mesh;
    }


    MC_mesh_index_vector MC_mesh_index_vector::subdivide_mixed_mid_edge_unchanged_boundary(const std::set<MC_int_pair,MC_int_pair_less>& allowed_edge) const
    {
        MC_mesh_index_vector new_mesh;
        new_mesh.point_set()=point_set();


        std::set <int> s=connectivity_mesh.boundary_polygon();
        std::set <MC_int_pair,MC_int_pair_less> bnd=connectivity_mesh.boundary();
        std::set <int> bnd_poly=connectivity_mesh.boundary_polygon();

        std::map <MC_int_pair,int,MC_int_pair_less> newly_added_vertices;

        MC_v3d_vector vertex=point_set_mesh;
        int N_polygon=polygon_number();
        for(int k=0;k<N_polygon;++k)
        {
            MC_int_vector poly=connectivity_mesh(k);
            std::pair<std::vector <MC_polygon>,std::pair<MC_mesh_index_vector,std::pair<MC_int_vector,MC_int_vector> > > current_poly;
            if(bnd_poly.find(k)==bnd_poly.end())//classical subdivision
                current_poly=get_polygon(k).subdivide_mixed_mid_edge();
            else
            {
                MC_int_vector constraint_edge;
                int poly_size=poly.size();
                for(int k_poly=0;k_poly<poly_size;++k_poly)
                {
                    MC_int_pair current_edge(poly[k_poly],poly[(k_poly+1)%poly_size]);
                    if(bnd.find(current_edge)!=bnd.end())
                        if(allowed_edge.find(current_edge)==allowed_edge.end())
                            constraint_edge.add(k_poly);
                }
                current_poly=get_polygon(k).subdivide_mixed_mid_edge(constraint_edge);
            }



            // goes back to global mesh
            MC_mesh_index_vector local_mesh=current_poly.second.first;
            MC_int_vector extra_vertices=current_poly.second.second.first;
            MC_int_vector edge_of_extra_vertices=current_poly.second.second.second;





            std::map <int,int> extra_local_to_global;

            //add the barycenter
            if(poly.size()!=3)
            {
                extra_local_to_global[poly.size()]=new_mesh.point_set().size();
                new_mesh.point_set().add(local_mesh.point_set()[poly.size()]);
            }

            for(int k_edge=0;k_edge<edge_of_extra_vertices.size();++k_edge)
            {
                MC_int_pair index_edge_local(edge_of_extra_vertices[k_edge],(edge_of_extra_vertices[k_edge]+1)%poly.size());
                MC_int_pair index_edge_global(poly(index_edge_local[0]),poly(index_edge_local[1]));


                std::pair <std::map<MC_int_pair,int,MC_int_pair_less>::iterator,bool> insert_it=newly_added_vertices.insert(std::make_pair(index_edge_global,new_mesh.point_set().size()));
                if(insert_it.second==true)
                {
                    extra_local_to_global[extra_vertices[k_edge]]=new_mesh.point_set().size();
                    new_mesh.point_set().add(local_mesh.point_set()[extra_vertices[k_edge]]);
                }
                else
                    extra_local_to_global[extra_vertices[k_edge]]=insert_it.first->second;
            }



            for(int k_new_poly=0;k_new_poly<local_mesh.polygon_number();++k_new_poly)
            {
                MC_int_vector local_poly=local_mesh.connectivity()(k_new_poly);
                MC_int_vector global_poly;
                for(int k_new_poly_vertex=0;k_new_poly_vertex<local_poly.size();++k_new_poly_vertex)
                {
                    int u=local_poly[k_new_poly_vertex];
                    if(u<poly.size())//old vertices
                        global_poly.add(poly[u]);
                    else
                        global_poly.add(extra_local_to_global[u]);

                }
                new_mesh.connectivity().add(global_poly);
            }
        }
        return new_mesh;
    }




    MC_mesh_index_vector MC_mesh_index_vector::subdivide_mid_edge() const
    {


        //the subdivided Mesh
        MC_mesh_index_vector mesh2;

        //MC_int_vector_vector index_mid_point(vertex_number); //the index of the mid point
        //MC_int_vector_vector already_existing_mid_point(vertex_number); //the already existing edges

        std::vector <std::map <int,int> > index_mid_point(vertex_number());


        //first add the old vertices
        mesh2.point_set()=point_set_mesh;

        // record the mid_points first
        int current_vertex=vertex_number();
        int N_polygon=polygon_number();
        for(int k_polygon=0;k_polygon<N_polygon;k_polygon++)
        {
            MC_int_vector index_poly=connectivity_mesh[k_polygon];
            int size_polygon=index_poly.size();
            for(int k_edge=0;k_edge<size_polygon;++k_edge)
            {
                int index_v0=index_poly[k_edge];
                int index_v1=index_poly[(k_edge+1)%size_polygon];

                if( index_mid_point[index_v0].find(index_v1)==index_mid_point[index_v0].end() )
                {
                    // connectivity save
                    index_mid_point[index_v0].insert(std::pair <int,int> (index_v1,current_vertex) );
                    index_mid_point[index_v1].insert(std::pair <int,int> (index_v0,current_vertex) );

                    // the geometrical mid_point
                    mesh2.point_set().add(0.5*(point_set_mesh(index_v0)+point_set_mesh(index_v1)));

                    current_vertex++;
                }
            }
        }

        // connectivity now
        for(int k_polygon=0;k_polygon<N_polygon;++k_polygon)
        {
            // the triangles/mid_edges-barycenter
            MC_int_vector index_poly=connectivity_mesh[k_polygon];
            int size_polygon=index_poly.size();

            for(int k_edge=0;k_edge<size_polygon;++k_edge)
            {
                int index_v0  = index_poly(k_edge);
                int index_v1  = index_poly((k_edge+1)%size_polygon);
                int index_vm1 = index_poly((k_edge+size_polygon-1)%size_polygon);

                int  index_mid  = index_mid_point[index_v0].find(index_v1)->second;
                int  index_mid2 = index_mid_point[index_v0].find(index_vm1)->second;

                mesh2.connectivity().add(MC_int_vector(index_v0,index_mid,index_mid2));
            }

            //add the non triangular part linking every barycenters
            MC_int_vector temp_poly;
            for(int k_edge=0;k_edge<size_polygon;++k_edge)
            {
                int index_v0  = index_poly(k_edge);
                int index_v1  = index_poly((k_edge+1)%size_polygon);

                int index_mid = index_mid_point[index_v0].find(index_v1)->second;
                temp_poly.add(index_mid);
            }
            mesh2.connectivity().add(temp_poly);
        }

        return mesh2;
    }


    MC_mesh_index_vector MC_mesh_index_vector::subdivide_barycenter_mid_edge() const
    {


        //the subdivided Mesh
        MC_mesh_index_vector mesh2;

        //MC_int_vector_vector index_mid_point(vertex_number); //the index of the mid point
        //MC_int_vector_vector already_existing_mid_point(vertex_number); //the already existing edges

        std::vector <std::map <int,int> > index_mid_point(vertex_number());
        std::map <int,int> barycenter_map;

        //first add the old vertices
        mesh2.point_set()=point_set_mesh;

        // record the mid_points first
        int current_vertex=vertex_number();
        int N_polygon=polygon_number();
        for(int k_polygon=0;k_polygon<N_polygon;k_polygon++)
        {
            MC_int_vector index_poly=connectivity_mesh[k_polygon];
            int size_polygon=index_poly.size();
            for(int k_edge=0;k_edge<size_polygon;++k_edge)
            {
                int index_v0=index_poly[k_edge];
                int index_v1=index_poly[(k_edge+1)%size_polygon];

                if( index_mid_point[index_v0].find(index_v1)==index_mid_point[index_v0].end() )
                {
                    // connectivity save
                    index_mid_point[index_v0].insert(std::pair <int,int> (index_v1,current_vertex) );
                    index_mid_point[index_v1].insert(std::pair <int,int> (index_v0,current_vertex) );

                    // the geometrical mid_point
                    mesh2.point_set().add(0.5*(point_set_mesh(index_v0)+point_set_mesh(index_v1)));

                    current_vertex++;
                }
            }

            // add the barycenter
            barycenter_map.insert(std::pair<int,int>(k_polygon,current_vertex++));
            mesh2.point_set().add(MC_v3d_vector::sum(mesh2.point_set()(index_poly))/size_polygon);
        }

        // connectivity now
        for(int k_polygon=0;k_polygon<N_polygon;++k_polygon)
        {
            // the triangles/mid_edges-barycenter
            MC_int_vector index_poly=connectivity_mesh[k_polygon];
            int size_polygon=index_poly.size();

            int index_barycenter=barycenter_map.find(k_polygon)->second;
            for(int k_edge=0;k_edge<size_polygon;++k_edge)
            {
                int index_v0  = index_poly(k_edge);
                int index_v1  = index_poly((k_edge+1)%size_polygon);
                int index_vm1 = index_poly((k_edge+size_polygon-1)%size_polygon);

                int  index_mid  = index_mid_point[index_v0].find(index_v1)->second;
                int  index_mid2 = index_mid_point[index_v0].find(index_vm1)->second;


                mesh2.connectivity().add(MC_int_vector(index_v0,index_mid,index_barycenter,index_mid2));
            }
        }

        return mesh2;
    }

    MC_mesh_index_vector MC_mesh_index_vector::build_sphere(const int& N_subdiv)
    {
        MC_mesh_index_vector new_sphere;
        new_sphere = MC_mesh_index_vector::build_icosahedron();

        int k=0;
        int N_vertices_1=0,N_vertices_2=0;

        //for every subdivision project on the unit sphere
        for(int k_subdiv=0;k_subdiv<N_subdiv;k_subdiv++)
        {
            N_vertices_1 = new_sphere.vertex_number();
            new_sphere=new_sphere.subdivide_mid_edge();
            N_vertices_2=new_sphere.vertex_number();
            for(k=N_vertices_1;k<N_vertices_2;k++)
                new_sphere.point_set()[k]=new_sphere.point_set()[k].normalized();
        }
        new_sphere.connectivity().update_neighbors();
        return new_sphere;
    }

    MC_mesh_index_vector MC_mesh_index_vector::build_quad_sphere(const int& N_subdiv)
    {
        MC_mesh_index_vector new_sphere;
        new_sphere = (MC_mesh_index_vector::build_cube()-0.5*MC_v3d(1,1,1))*2/sqrt(3);

        int k=0;
        int N_vertices_1=0,N_vertices_2=0;

        //for every subdivision project on the unit sphere
        for(int k_subdiv=0;k_subdiv<N_subdiv;k_subdiv++)
        {
            N_vertices_1 = new_sphere.vertex_number();
            new_sphere=new_sphere.subdivide_barycenter_mid_edge();
            N_vertices_2=new_sphere.vertex_number();
            for(k=N_vertices_1;k<N_vertices_2;k++)
                new_sphere.point_set()[k]=new_sphere.point_set()[k].normalized();
        }
        return new_sphere;

    }

    MC_mesh_index_vector MC_mesh_index_vector::build_parametric_sphere(const int& N_1,const int& N_2)
    {
        MC_mesh_index_vector new_sphere;
#define PI 3.14159

        int k1=0,k2=0;
        double u1=0.0,u2=0.0;
        for(k1=0;k1<N_1;k1++)
        {
            u1=double(k1+1)/double(N_1+1);
            for(k2=0;k2<N_2;k2++)
            {
                u2=double(k2)/double(N_2);

                new_sphere.add_vertex(MC_v3d(sin(PI*u1)*cos(2*PI*u2),sin(PI*u1)*sin(2*PI*u2),cos(PI*u1)));
                if(k1<N_1-1)
                    new_sphere.connectivity().add(MC_int_vector(k2         +N_2*k1 ,
                                                                (k2+1)%N_2 +N_2*k1 ,
                                                                (k2+1)%N_2 +N_2*( (k1+1)%N_1 ),
                                                                k2         +N_2*( (k1+1)%N_1 )));
            }
        }

        //starting and ending point
        new_sphere.point_set().add(MC_v3d(0,0,1));
        new_sphere.point_set().add(MC_v3d(0,0,-1));


        //do triangle for the two extremities
        for(k2=0;k2<N_2;k2++)
        {
            new_sphere.connectivity().add(MC_int_vector(
                    N_1*N_2,
                    k2,
                    (k2+1)%N_2
                    ));
            new_sphere.connectivity().add(MC_int_vector(
                    N_1*N_2+1,
                    k2+N_2*(N_1-1),
                    (k2+1)%N_2+N_2*(N_1-1)
                    ));
        }

        return new_sphere;
    }

    MC_mesh_index_vector MC_mesh_index_vector::build_torus(const double& R0,const double& R1,const int& N_1,const int& N_2)
    {

        MC_mesh_index_vector new_torus;
#define PI 3.14159

        int k1=0,k2=0;
        double u1=0.0,u2=0.0;
        for(k1=0;k1<N_1;k1++)
        {
            u1=double(k1)/double(N_1);
            for(k2=0;k2<N_2;k2++)
            {
                u2=double(k2)/double(N_2);

                new_torus.point_set().add(MC_v3d(
                        (R0+R1*cos(u1*2*PI))*cos(u2*2*PI),
                        (R0+R1*cos(u1*2*PI))*sin(u2*2*PI),
                        R1*sin(u1*2*PI)
                        ));

                new_torus.connectivity().add(MC_int_vector(
                        k2         +N_2*k1 ,
                        (k2+1)%N_2 +N_2*k1 ,
                        (k2+1)%N_2 +N_2*( (k1+1)%N_1 ),
                        k2         +N_2*( (k1+1)%N_1 )));
            }
        }

        return new_torus;

    }



    MC_mesh_index_vector MC_mesh_index_vector::build_closed_cylinder(const int& N1,const int& N2,const bool& is_closed)
    {
          MC_mesh_index_vector cylinder;


          double u1=0.0,u2=0.0;
          int k1=0,k2=0;
          for(k1=0;k1<N1;k1++)
          {
              u1 = double(k1)/double(N1-1);
              for(k2=0;k2<N2;k2++)
              {
                  u2 = double(k2)/double(N2);
                  cylinder.point_set().add(MC_v3d(u1,cos(2*PI*u2),sin(2*PI*u2)));
              }
          }

          for(k1=0;k1<N1-1;k1++)
              for(k2=0;k2<N2;k2++)
                  cylinder.connectivity().add(MC_int_vector( k2+0     + N2*k1+0,
                                                               (k2+1)%N2 + N2*k1+0,
                                                               (k2+1)%N2 + N2*(k1+1),
                                                               k2+0     + N2*(k1+1)));


          if(is_closed==true)
          {
              // to close the cylinder
              cylinder.add_vertex(MC_v3d(0,0,0));
              cylinder.add_vertex(MC_v3d(1,0,0));

              MC_int_vector closing;
              for(k2=0;k2<N2;++k2)
                  cylinder.add_connectivity_index(MC_int_vector(k2,N2*N1,(k2+1)%N2));
              for(k2=0;k2<N2;++k2)
                  cylinder.add_connectivity_index(MC_int_vector(N2*N1+1,N2*(N1-1)+k2,N2*(N1-1)+(k2+1)%N2));
          }

          return cylinder;

    }

    MC_mesh_index_vector MC_mesh_index_vector::build_cone(const int& N_radius,const int& N_face)
    {

#define PI 3.14159

        //special cone
        if(N_radius==4 && N_face==2)
        {
            MC_mesh_index_vector cone;
            cone.point_set()=MC_v3d_vector(MC_v3d(1,1,0))<<MC_v3d(1,-1,0)<<MC_v3d(-1,-1,0)<<MC_v3d(-1,1,0)<<MC_v3d(0,0,1);
            cone.connectivity().add(MC_int_vector(0,3,2,1));
            cone.connectivity().add(MC_int_vector(0,1,4));
            cone.connectivity().add(MC_int_vector(1,2,4));
            cone.connectivity().add(MC_int_vector(2,3,4));
            cone.connectivity().add(MC_int_vector(3,0,4));

            return cone;
        }

        MC_mesh_index_vector cone;

        // add the two extreme points
        cone.add_vertex(MC_v3d(0,0,0));
        cone.add_vertex(MC_v3d(0,0,1));

        // vertices
        double r=1.0/(N_face-1);
        for(int k=0;k<N_radius;k++)
        {
            double alpha = 2*PI*double(k)/double(N_radius);
            MC_v3d x=MC_v3d(r*cos(alpha),r*sin(alpha),0.0);
            cone.add_vertex(x);
        }
        //base of cone
        for(int k2=1;k2<N_face;k2++)
        {
            double r=double(k2)/double(N_face-1);
            for(int k=0;k<N_radius;k++)
            {
                double alpha = 2*PI*double(k)/double(N_radius);
                MC_v3d x = MC_v3d(r*cos(alpha),r*sin(alpha),0.0);
                cone.add_vertex(x);
            }
        }
        //triangle cone
        for(int k2=1;k2<N_face-1;k2++)
        {
            double r=double(N_face-1-k2)/double(N_face-1);
            double h=double(k2)/double(N_face-1);
            for(int k=0;k<N_radius;k++)
            {
                double alpha = 2*PI*double(k)/double(N_radius);
                MC_v3d x=MC_v3d(r*cos(alpha),r*sin(alpha),h);
                cone.add_vertex(x);
            }
        }


        // connectivity
        for(int k=0;k<N_radius;k++)
            cone.add_connectivity_index(MC_int_vector(0,(k+1)%N_radius+2,k+2));
        for(int k2=1;k2<N_face-1;k2++)
        {
            for(int k=0;k<N_radius;k++)
            {
                cone.add_connectivity_index(MC_int_vector((k+0)  +(k2+0)*N_radius+2,
                                               (k+1)%N_radius+(k2+0)*N_radius+2,
                                               (k+1)%N_radius+(k2+1)*N_radius+2,
                                               (k+0)  +(k2+1)*N_radius+2));
            }
        }

        ///
        for(int k2=0;k2<N_face-2;k2++)
            for(int k=0;k<N_radius;k++)
            {
            cone.add_connectivity_index(MC_int_vector((k+0)  +(k2+0)*N_radius+N_radius*(N_face-1)+2,
                                           (k+1)%N_radius+(k2+0)*N_radius+N_radius*(N_face-1)+2,
                                           (k+1)%N_radius+(k2+1)*N_radius+N_radius*(N_face-1)+2,
                                           (k+0)  +(k2+1)*N_radius+N_radius*(N_face-1)+2));
        }

        /// triangles of the hat
        for(int k=0;k<N_radius;k++)
        {
            cone.add_connectivity_index(MC_int_vector( k+N_radius*(N_face-2)+N_radius*(N_face-1)+2,
                                            (k+1)%N_radius+N_radius*(N_face-2)+N_radius*(N_face-1)+2,
                                            1));
        }



        return cone;
    }

    MC_mesh_index_vector MC_mesh_index_vector::build_disc(const MC_v3d& center,const MC_v3d& normal,const double& radius,const int& N_radius,const int& N_interior)
    {
        double epsilon=0.0001;
        if(normal.norm()<epsilon)
        {std::cout<<"Error in MC_mesh_index_vector::build_disc(...), normal is null"<<std::endl;exit(-1);}

        MC_v3d axis_1 = normal.cross(MC_v3d(1,0,0));
        if(axis_1.norm()<epsilon)
        {
            axis_1 = normal.cross(MC_v3d(0,1,0));
            if(axis_1.norm()<epsilon)
            {std::cout<<"Error in MC_mesh_index_vector::build_disc, something weird"<<std::endl; exit(-1);}
        }

        axis_1 = axis_1.normalized();
        MC_v3d axis_2 = (axis_1.cross(normal)).normalized();

        MC_mesh_index_vector disc;

        // vertices
        disc.add_vertex(center);
        for(int k_radius=0;k_radius<N_radius;k_radius++)
        {
            double theta = 2*PI*double(k_radius)/double(N_radius);
            for(int k_interior=1;k_interior<N_interior;k_interior++)
            {
                double r=radius*double(k_interior)/double(N_interior-1);

                MC_v3d current=r*cos(theta)*axis_1 + r*sin(theta)*axis_2;
                disc.add_vertex(center+current);
            }
        }


        // connectivity
        for(int k_radius=0;k_radius<N_radius;k_radius++)
        {
            for(int k_interior=0;k_interior<N_interior-1;k_interior++)
            {
                if(k_interior==0)
                    disc.add_connectivity_index(MC_int_vector( 0,k_radius*(N_interior-1)+1,((k_radius+1)%N_radius)*(N_interior-1)+1 ));
                else
                    disc.add_connectivity_index(MC_int_vector((k_interior-1+0)+(k_radius+0)*(N_interior-1)+1,
                                                              (k_interior-1+1)+(k_radius+0)*(N_interior-1)+1,
                                                              (k_interior-1+1)+((k_radius+1)%N_radius)*(N_interior-1)+1,
                                                              (k_interior-1+0)+((k_radius+1)%N_radius)*(N_interior-1)+1 ));
            }
        }


        return disc;
    }

    MC_mesh_index_vector MC_mesh_index_vector::build_arrow(const MC_segment& dir,const double& radius_cylinder,const int& N_cylinder,const int& N_shape_cylinder,const double& radius_cone,const double& length_cone,const int& N_cone,const int& N_shape_cone)
    {

        // protection against singularity
        double epsilon=0.0001;

        MC_mesh_index_vector arrow;
        MC_v3d V_intermediate = dir.unit_vector()*(dir.length()-length_cone);

        MC_mesh_index_vector cylinder = MC_mesh_index_vector::build_closed_cylinder(N_shape_cylinder,N_cylinder);
        cylinder.point_set().scale(MC_v3d(V_intermediate.norm(),radius_cylinder,radius_cylinder));

        MC_matrix R=MC_matrix::identity(3);
        if(dir.unit_vector().cross(MC_v3d(1,0,0)).norm()>epsilon)
            R=MC_matrix::rotation_axis_to_axis(MC_v3d(1,0,0),dir.unit_vector());
        cylinder = R*cylinder;

        MC_mesh_index_vector cone = MC_mesh_index_vector::build_cone(N_cone,N_shape_cone);
        cone.point_set().scale(MC_v3d(radius_cone,radius_cone,length_cone));

        R=MC_matrix::identity(3);
        if(dir.unit_vector().cross(MC_v3d(0,0,1)).norm()>epsilon)
            R=MC_matrix::rotation_axis_to_axis(MC_v3d(0,0,1),dir.unit_vector()+MC_v3d(epsilon,0,0));
        cone=R*cone+V_intermediate;


        arrow = cone<<cylinder;
        arrow+=dir[0];

        return arrow;


    }

    MC_mesh_index_vector operator<<(const MC_mesh_index_vector& vec0,const MC_mesh_index_vector& vec1)
    {
        MC_mesh_index_vector new_vec;

        new_vec.point_set()=vec0.point_set()<<vec1.point_set();
        new_vec.connectivity()=vec0.connectivity()<<(vec1.connectivity()+vec0.vertex_number());

        return new_vec;
    }


    MC_mesh_index_vector& MC_mesh_index_vector::concatenation(const MC_mesh_index_vector& vec1)
    {
        int N_vertex=point_set().size();
        point_set().add(vec1.point_set());
        connectivity().add(vec1.connectivity()+N_vertex);
        return *this;
    }

    std::pair <std::vector <MC_curve>,MC_int_vector_vector> MC_mesh_index_vector::boundary_curve() const
    {
        // get the boundary index
        MC_int_vector_vector full_index(MC_int_vector::connected_component(connectivity_mesh.boundary()));

        // fill the curves
        int N=full_index.size();
        std::vector <MC_curve> full_curve(N);
        for(int k=0;k<N;++k)
            full_curve[k]=point_set_mesh(full_index(k));

        return std::pair <std::vector <MC_curve>,MC_int_vector_vector> (full_curve,full_index);

    }


    std::vector <MC_curve> MC_mesh_index_vector::plane_intersection(const MC_v3d& n,const MC_v3d& x0) const
    {
        std::vector <MC_curve> curves;
        int k_polygon=0,N_polygon=polygon_number();

        std::vector <MC_segment> s;int type=-1;
        MC_v3d_vector temp;
        //check every intersection for every polygons
        //store the intersection in an unordered vector of Segment
        for(k_polygon=0;k_polygon<N_polygon;k_polygon++)
        {
            //check the intersection
            temp = get_polygon(k_polygon).plane_intersection(n,x0,&type);

            if(type==1 || type==3)
            {
               // std::cout<<k_polygon<<" ! "<<temp<<std::endl;
                s.push_back(MC_segment(temp[0],temp[1]));
            }
        }


        //avoid the identical segments
        std::vector<MC_segment> s2=s;
        s.clear();
        for(int k=0,N=s2.size();k<N;++k)
        {
            bool is_unique=true;
            for(int k2=0;is_unique==true && k2<s.size();++k2)
            {
                if( (s2[k][0]==s[k2][0] && s2[k][1]==s[k2][1])
                    ||
                    (s2[k][1]==s[k2][0] && s2[k][0]==s[k2][1])
                    )
                    is_unique=false;
            }
            if(is_unique==true)
                s.push_back(s2[k]);
        }




        int segment_number=s.size();
        if(segment_number==0)
            return curves;


        // now order the segments in curve
        MC_int_vector is_segment_added;
        is_segment_added.resize(segment_number);

        std::list <MC_v3d> current_curve;
        //MC_curve current_curve;
        current_curve.push_back(s[0][0]);
        current_curve.push_back(s[0][1]);
        is_segment_added[0]=1;

//        std::cout<<segment_number<<std::endl;
//        for(int k=0;k<segment_number;++k)
//            std::cout<<s[k]<<std::endl;

        int k_segment=-1,is_addition=1;
        while(is_addition==1)//as long as there is segment to add
        {
            //check if there is at least one added segment during the whole pass
            is_addition=0;

            //add forward
            for(k_segment=0;k_segment<segment_number;k_segment++){
                if(is_segment_added[k_segment]==0)
                {
                    if(s[k_segment][0]==current_curve.back())
                    {current_curve.push_back(s[k_segment][1]);is_segment_added[k_segment]=1;is_addition=1;}
                    else if(s[k_segment][1]==current_curve.back())
                    {current_curve.push_back(s[k_segment][0]);is_segment_added[k_segment]=1;is_addition=1;}
                }
            }
            //add back
            for(k_segment=0;k_segment<segment_number;k_segment++){
                if(is_segment_added[k_segment]==0)
                {
                    if(s[k_segment][0]==current_curve.front())
                    {current_curve.push_front(s[k_segment][1]);is_segment_added[k_segment]=1;is_addition=1;}
                    else if(s[k_segment][1]==current_curve.front())
                    {current_curve.push_front(s[k_segment][0]);is_segment_added[k_segment]=1;is_addition=1;}
                }
            }

                //if there is no added segment, it might be non-connected cases
                if(is_addition==0)
                {
                    // add this curve to the vector
                    curves.push_back(current_curve);
                    current_curve.clear();

                    //check if every segment has been added
                    for(k_segment=0;is_addition==0 && k_segment<segment_number;k_segment++){
                        if(is_segment_added[k_segment]==0)//Then add it in the next curve
                        {
                            is_addition=1;
                            current_curve.push_back(s[k_segment][0]);current_curve.push_back(s[k_segment][1]);
                            is_segment_added[k_segment]=1;
                        }
                    }
                }


        }
        return curves;
    }

    MC_mesh_index_vector MC_mesh_index_vector::half_space_intersection(const MC_v3d& n,const MC_v3d& x0,int *type) const
    {

        int k_polygon=0;
        int N_polygon=polygon_number();

        std::vector <MC_polygon> polygon_soup;

        if(type!=0)
            *type = 0;
        MC_polygon p; int t=0;
        MC_polygon q;

        //cut every polygon
        for(k_polygon=0;k_polygon<N_polygon;++k_polygon)
        {
            p = get_polygon(k_polygon);
            q = p.half_space_intersection(n,x0,&t);

            if(t!=2)//add the polygon and build the connectivity
                polygon_soup.push_back(q);
            if(t!=0)//at least one polygon is cutted
                if(type!=0)
                    *type=1;


        }

        MC_mesh_index_vector new_mesh=polygon_soup;

        if(new_mesh.vertex_number()==0)//no more mesh at all
            if(type!=0)
                *type=2;


        return new_mesh;
    }

    std::pair<MC_mesh_index_vector,std::pair<MC_int_vector_vector,MC_int_vector_vector> > MC_mesh_index_vector::added_polygon_soup(const std::vector<MC_polygon> poly_soup) const
    {
        MC_mesh_index_vector current_mesh=*this;
        std::map <MC_v3d,int,MC_v3d_less> v_3d_map = point_set().to_map();

        std::pair <MC_int_vector_vector,MC_int_vector_vector> res;
        std::pair <MC_int_vector,MC_int_vector> temp;

        int N_poly=poly_soup.size();
        for(int k=0;k<N_poly;k++)
        {
            bool is_added=true;
            MC_polygon p=poly_soup[k];
            if(p.is_degenerated()==true)
            {
                std::pair <MC_polygon,std::pair<bool,bool> > up=p.undegenerated();
                if(up.second.second==false)
                    is_added=false;
                else
                    p=up.first;
            }
            if(is_added==true)
            {
                temp=current_mesh.add_unique_polygon(p,&v_3d_map);
                res.first.add(temp.first); res.second.add(temp.second);
            }
        }

        return std::pair<MC_mesh_index_vector,std::pair<MC_int_vector_vector,MC_int_vector_vector> > (current_mesh,res);
    }

    std::pair <MC_int_vector,MC_int_vector> MC_mesh_index_vector::add_unique_polygon(const MC_polygon& polygon,std::map <MC_v3d,int,MC_v3d_less>* map_vertices)
    {
        MC_int_vector index_polygon;
        MC_int_vector is_new;


        std::map <MC_v3d,int,MC_v3d_less> :: iterator it;

        int N_polygon=polygon.size();
        for(int k=0;k<N_polygon;++k)
        {
            std::pair<std::map <MC_v3d,int,MC_v3d_less>::iterator ,bool> it=map_vertices->insert(std::pair<MC_v3d,int>(polygon[k],vertex_number()));

            if(it.second==true)
            {
                point_set().add(polygon[k]);
                index_polygon.add(vertex_number()-1);
                is_new.add(1);
            }
            else //already exists
            {
                index_polygon.add(it.first->second);
                is_new.add(0);
            }
        }

        connectivity().add(index_polygon);
        return std::pair < MC_int_vector , MC_int_vector > (index_polygon,is_new);
    }

    MC_mesh_index_vector::MC_mesh_index_vector(const std::vector<MC_polygon> poly_soup)
    {
        *this=this->added_polygon_soup(poly_soup).first;
    }

    std::pair <MC_v3d_vector,std::pair <MC_int_vector,MC_double_vector> > MC_mesh_index_vector::segment_intersection(const MC_segment& s) const
    {
        MC_int_vector intersected_polygon;
        MC_double_vector segment_coord;
        MC_v3d_vector inter;
        int N=polygon_number();
        MC_polygon p;
        int type=0;
        for(int k=0;k<N;k++)
        {
            p=get_polygon(k);
            MC_v3d i = p.segment_intersection(s,&type);

            if(type==1 || type==2 || type==3)
            {
                inter.add(i);
                intersected_polygon.add(k);

                bool is_alignated=false;
                segment_coord.add(s.relative_position(i,&is_alignated));
                if(is_alignated==false)
                {std::cout<<"Something wrong in MC_mesh_index_vector::segment_intersection()"<<std::endl;exit(-1);}
            }
        }

        // ordonate the intersection from begining to end of the curve
        std::map <double,int> map_order=segment_coord.to_map();
        std::map <double,int>::const_iterator it=map_order.begin(),it_end=map_order.end();

        std::pair <MC_v3d_vector,std::pair <MC_int_vector,MC_double_vector> > res;
        for(;it!=it_end;++it)
        {
            int index=it->second;
            res.first.add(inter[index]);
            res.second.first.add(intersected_polygon[index]);
            res.second.second.add(segment_coord[index]);
        }

        return res;
    }

    std::pair <MC_int_vector,std::pair <MC_v3d_vector,std::pair <MC_int_vector,MC_double_vector> > >
            MC_mesh_index_vector::segment_intersection(const std::vector <MC_mesh_index_vector>& v_mesh,const MC_segment& s)
    {
        std::vector <std::pair <MC_v3d_vector,std::pair <MC_int_vector,MC_double_vector> > > v_inter;
        int N=v_mesh.size();

        //keep all the data intersection from the data
        for(int k=0;k<N;++k)
            v_inter.push_back(v_mesh[k].segment_intersection(s));

        // merge everything now
        MC_int_vector temp_index;
        MC_v3d_vector temp_inter;
        MC_int_vector temp_poly;
        MC_double_vector temp_relative;
        for(int k=0;k<N;++k)
        {
            if(v_inter[k].first.size()>0)
            {
                temp_index.add(MC_int_vector::zeros(v_inter[k].first.size())+k);
                temp_inter.add(v_inter[k].first);
                temp_poly.add(v_inter[k].second.first);
                temp_relative.add(v_inter[k].second.second);
            }
        }


        // ordonate it
        std::pair <MC_double_vector,MC_int_vector> ind=temp_relative.sort();

        std::pair <MC_int_vector,std::pair <MC_v3d_vector,std::pair <MC_int_vector,MC_double_vector> > > res;
        res.first=temp_index(ind.second);
        res.second.first=temp_inter(ind.second);
        res.second.second.first=temp_poly(ind.second);
        res.second.second.second=ind.first;

        return res;
    }
    MC_mesh_index_vector MC_mesh_index_vector::sweep_surface(const MC_curve& _c,const MC_curve& _pattern,const int& N_subdiv,const bool& is_closed,const MC_v3d_vector& e1)
    {

        MC_curve c=_c.value_t(MC_double_vector::linspace(0,_c.size()-1, 1.0/static_cast<double>(N_subdiv-1)) ).first;

        bool is_given_tangent=(c.size()==e1.size());

        MC_curve pattern=_pattern;
        if(pattern.size()==0)
            pattern=MC_curve::build_circle(0.1,8,MC_v3d(0,0,1));

        int N_curve=c.size();
        int k_curve=0;

        MC_mesh_index_vector sweep;
        int N_pattern=pattern.size();


        MC_v3d_vector normal=c.diff_forward();


        // first rotation to the normal to the curve
        MC_matrix R0    =MC_matrix::identity(3);
        MC_matrix R0_old=MC_matrix::identity(3);
        //second rotation to avoid the twist
        MC_matrix R1    =MC_matrix::identity(3);
        MC_matrix R1_old=MC_matrix::identity(3);

        //the pattern during transformation
        MC_curve temp_pattern;
        int k_pattern=0;



        MC_v3d ez(0,0,1),ex(1,0,0),ey(0,1,0);
        MC_v3d e0x,e1x,e1y,e0p;
        for(k_curve=0;k_curve<N_curve;++k_curve)
        {
            //******************************//
            // Rotation
            //******************************//


            //first rotate to be aligned with the normal
            R0_old=R0;
            R0=MC_matrix::rotation_axis_to_axis(ez,normal[k_curve]);
            temp_pattern = R0*pattern;


            //second, avoid the twist
            if(is_given_tangent==false)
            {
                if(k_curve>=1)
                {

                    e0x = R1_old*R0_old*ex;

                    e1x = R0*ex;
                    e1y = R0*ey;

                    e0p = (e0x.dot(e1x))*e1x+(e0x.dot(e1y))*e1y;



                    // how to rotate the new projection ex to the old one
                    R1=MC_matrix::rotation_axis_to_axis(e1x,e0p);
                    temp_pattern = R1*temp_pattern;

                }
            }
            else
            {
                e1x = R0*ex;
                R1=MC_matrix::rotation_axis_to_axis(e1x,e1[k_curve]);
                temp_pattern = R1*temp_pattern;
            }


            //translate to the position of the curve
            temp_pattern=temp_pattern+c[k_curve];


            //save the old transformation
            R0_old=R0;R1_old=R1;




            //******************************//
            // set up the Mesh
            //******************************//


            /// add the vertices
            sweep.point_set().add(temp_pattern);

            /// add the quadrilateral faces
            if(k_curve<N_curve-1)
            {
                for(k_pattern=0;k_pattern<N_pattern-1;k_pattern++)
                {
                    sweep.connectivity().add(MC_int_vector(k_pattern+N_pattern*(k_curve),
                                                           k_pattern+N_pattern*(k_curve+1),
                                                           k_pattern+1+N_pattern*(k_curve+1),
                                                           k_pattern+1+N_pattern*(k_curve)
                                                           ));
                }
                //close the pattern
                sweep.connectivity().add(MC_int_vector(k_pattern+N_pattern*(k_curve),
                                                       k_pattern+N_pattern*(k_curve+1),
                                                       0+N_pattern*(k_curve+1),
                                                       0+N_pattern*(k_curve)
                                                       ));
            }



        }







        if(is_closed==true)
        {

              int k_subdiv=0;
              double s=0.0;
              int offset=0;

              R0    =MC_matrix::identity(3);
              R0_old=MC_matrix::identity(3);
              R1    =MC_matrix::identity(3);
              R1_old=MC_matrix::identity(3);


              MC_v3d barycenter;
              for(k_curve=0;k_curve<N_curve;k_curve++)
              {

                  //first rotate to be aligned with the normal
                  R0_old=R0;
                  R0=MC_matrix::rotation_axis_to_axis(ez,normal[k_curve]);
                  temp_pattern = R0*pattern;

                  //second, avoid the twist
                  if(is_given_tangent==false)
                  {
                      if(k_curve>=1)
                      {
                          e0x = R1_old*R0_old*ex;

                          e1x = R0*ex;
                          e1y = R0*ey;

                          e0p = (e0x.dot(e1x))*e1x+(e0x.dot(e1y))*e1y;

                          // how to rotate the new projection ex to the old one
                          R1=MC_matrix::rotation_axis_to_axis(e1x,e0p);
                          temp_pattern = R1*temp_pattern;
                      }
                  }
                  else
                  {
                      e1x = R0*ex;
                      R1=MC_matrix::rotation_axis_to_axis(e1x,e1[k_curve]);
                      temp_pattern = R1*temp_pattern;
                  }

                  //save the old transformation
                  R0_old=R0;R1_old=R1;



                  //******************************//
                  // set up the Mesh
                  //******************************//


                  /// add the vertices
                  if(k_curve==0 || k_curve==N_curve-1)
                  {
                      if(k_curve==0)
                          offset=N_curve*N_pattern;
                      if(k_curve==N_curve-1)
                          offset=N_curve*N_pattern+(N_subdiv-1)*N_pattern+1;//+1 due to the barycenter
                      for(k_subdiv=0;k_subdiv<N_subdiv-1;k_subdiv++)
                      {
                          s = double(k_subdiv+1)/double(N_subdiv+1);
                          sweep.point_set().add((1-s)*temp_pattern+c[k_curve]);

                          // add the quadrilateral faces
                          if(k_subdiv<N_subdiv-2)
                              for(k_pattern=0;k_pattern<N_pattern;k_pattern++)
                              {
                              if(k_curve==0)
                                  sweep.connectivity().add(MC_int_vector(offset + k_pattern  +N_pattern*(k_subdiv  ),
                                                                         offset + (k_pattern+1)%N_pattern+N_pattern*(k_subdiv  ),
                                                                         offset + (k_pattern+1)%N_pattern+N_pattern*(k_subdiv+1),
                                                                         offset + k_pattern  +N_pattern*(k_subdiv+1)
                                                                         ));
                              else//to be manifold
                                  sweep.connectivity().add(MC_int_vector(offset + k_pattern  +N_pattern*(k_subdiv  ),
                                                                         offset + k_pattern  +N_pattern*(k_subdiv+1),
                                                                         offset + (k_pattern+1)%N_pattern+N_pattern*(k_subdiv+1),
                                                                         offset + (k_pattern+1)%N_pattern+N_pattern*(k_subdiv  )
                                                                         ));
                          }
                      }
                      //close with the boundaries
                      for(k_pattern=0;k_pattern<N_pattern;k_pattern++)
                      {
                          if(k_curve==0)
                              sweep.connectivity().add(MC_int_vector(k_pattern                        +N_pattern*k_curve,
                                                                     (k_pattern+1)%N_pattern          +N_pattern*k_curve,
                                                                     offset + (k_pattern+1)%N_pattern                   ,
                                                                     offset + k_pattern
                                                                     ));
                          else//to be manifold
                              sweep.connectivity().add(MC_int_vector(k_pattern                        +N_pattern*k_curve,
                                                                     offset + k_pattern                                 ,
                                                                     offset + (k_pattern+1)%N_pattern                   ,
                                                                     (k_pattern+1)%N_pattern          +N_pattern*k_curve
                                                                     ));
                      }

                      //Now close with the midpoint
                      barycenter = (temp_pattern+c[k_curve]).barycenter();

                      // add the barycenter
                      sweep.add_vertex(barycenter);

                      //now close the surface
                      for(k_pattern=0;k_pattern<N_pattern;k_pattern++)
                      {
                          if(k_curve==0)
                              sweep.connectivity().add(MC_int_vector(offset + k_pattern + N_pattern*(N_subdiv-2),
                                                                     offset + (k_pattern+1)%N_pattern + N_pattern*(N_subdiv-2),
                                                                     offset + N_pattern*(N_subdiv-1)
                                                                     ));
                          else//to be a manifold mesh
                              sweep.connectivity().add(MC_int_vector(offset + k_pattern + N_pattern*(N_subdiv-2),
                                                                     offset + N_pattern*(N_subdiv-1),
                                                                     offset + (k_pattern+1)%N_pattern + N_pattern*(N_subdiv-2)
                                                                     ));
                      }
                  }
              }

          }



        return sweep;
    }
    double MC_mesh_index_vector::volume() const
    {
        double vol=0.0;
        int N_polygon=polygon_number();
        for(int k_polygon=0;k_polygon<N_polygon;++k_polygon)
        {
            MC_int_vector index_polygon=connectivity()(k_polygon);
            int N_vertex=index_polygon.size();
            int N_tri=N_vertex-2;
            for(int k_tri=0;k_tri<N_tri;++k_tri)
            {
                MC_v3d x0=point_set()(index_polygon[0]);
                MC_v3d x1=point_set()(index_polygon[k_tri+1]);
                MC_v3d x2=point_set()(index_polygon[k_tri+2]);

                vol+= (x0[2]+x1[2]+x2[2])*
                      ((x1[0]-x0[0])*(x2[1]-x0[1])-(x1[1]-x0[1])*(x2[0]-x0[0]));
            }
        }
        vol/=6.0;
        return vol;
    }
    MC_v3d_vector MC_mesh_index_vector::volume_gradient() const
    {
        int N_vertex=vertex_number();
        MC_v3d_vector grad(N_vertex);
        int N_polygon=polygon_number();

        for(int k_polygon=0;k_polygon<N_polygon;++k_polygon)
        {
            MC_int_vector index_polygon=connectivity()(k_polygon);
            int N_vertex=index_polygon.size();
            int N_tri=N_vertex-2;
            for(int k_tri=0;k_tri<N_tri;++k_tri)
            {
                int index_0=index_polygon[0];
                int index_1=index_polygon[k_tri+1];
                int index_2=index_polygon[k_tri+2];

                MC_v3d x0=point_set()(index_0);
                MC_v3d x1=point_set()(index_1);
                MC_v3d x2=point_set()(index_2);

                grad[index_0][0] += 0.5* (x0[2]+x1[2]+x2[2]) * (x1[1]-x2[1]) * 1/3.0;
                grad[index_1][0] += 0.5* (x0[2]+x1[2]+x2[2]) * (x2[1]-x0[1]) * 1/3.0;
                grad[index_2][0] += 0.5* (x0[2]+x1[2]+x2[2]) * (x0[1]-x1[1]) * 1/3.0;

                grad[index_0][1] += 0.5* (x0[2]+x1[2]+x2[2]) * (x2[0]-x1[0]) * 1/3.0;
                grad[index_1][1] += 0.5* (x0[2]+x1[2]+x2[2]) * (x0[0]-x2[0]) * 1/3.0;
                grad[index_2][1] += 0.5* (x0[2]+x1[2]+x2[2]) * (x1[0]-x0[0]) * 1/3.0;

                grad[index_0][2] += 0.5*(x1[0]*x2[1]-x1[0]*x0[1]-x0[0]*x2[1]-x1[1]*x2[0]+x1[1]*x0[0]+x0[1]*x2[0]) * 1/3.0;
                grad[index_1][2] += 0.5*(x1[0]*x2[1]-x1[0]*x0[1]-x0[0]*x2[1]-x1[1]*x2[0]+x1[1]*x0[0]+x0[1]*x2[0]) * 1/3.0;
                grad[index_2][2] += 0.5*(x1[0]*x2[1]-x1[0]*x0[1]-x0[0]*x2[1]-x1[1]*x2[0]+x1[1]*x0[0]+x0[1]*x2[0]) * 1/3.0;
            }
        }

        return grad;
    }




    MC_mesh_index_vector MC_mesh_index_vector::build_square(const int& N_1,const int& N_2)
    {

        MC_mesh_index_vector mesh;

        for(int k_1=0;k_1<N_1+2;++k_1){
            double alpha_1=static_cast<double>(k_1)/static_cast<double>(N_1+1);
            for(int k_2=0;k_2<N_2+2;++k_2){
                double alpha_2=static_cast<double>(k_2)/static_cast<double>(N_2+1);
                mesh.point_set().add(MC_v3d(alpha_1,alpha_2,0.0));
            }
        }

        for(int k_1=0;k_1<N_1+2-1;++k_1){
            for(int k_2=0;k_2<N_2+2-1;++k_2){
                mesh.connectivity().add(MC_int_vector((k_1+0)*(N_2+2)+(k_2+0),
                                                      (k_1+0)*(N_2+2)+(k_2+1),
                                                      (k_1+1)*(N_2+2)+(k_2+1),
                                                      (k_1+1)*(N_2+2)+(k_2+0)));
            }
        }

        return mesh;
    }












    MC_double_vector_vector MC_mesh_index_vector::barycentric_coordinates(const MC_mesh_index_vector& mesh,const MC_v3d_vector& vertices,const MC_int_vector& belonging_polygon)
    {
        if(belonging_polygon.size()!=vertices.size())
        {std::cout<<"Error in MC_mesh_index_vector::barycentric_coordinates(), size are not compatible"<<std::endl;exit(-1);}

        int N=vertices.size();
        int N_poly=mesh.polygon_number();
        MC_double_vector_vector bar=MC_double_vector_vector::empty(N);
        for(int k=0;k<N;++k)
        {
            int index_poly=belonging_polygon(k);
            if(index_poly<0 || index_poly>N_poly)
            {std::cout<<"Error in MC_mesh_index_vector::barycentric_coordinates() at index k="<<k<<"/"<<N<<" not correct with N_polygon="<<N_poly<<std::endl;exit(-1);}
            bar[k]=mesh.get_polygon(index_poly).barycentric_coordinates(vertices[k]);
        }
        return bar;
    }



    std::pair <std::pair<MC_mesh_index_vector,MC_mesh_index_vector>,std::pair<MC_int_vector,MC_int_vector> > MC_mesh_index_vector::delete_polygon(const std::set<int> index_to_delete) const
    {
        int N_poly=polygon_number();

        std::set <int> :: const_iterator index_to_delete_end=index_to_delete.end();

        std::map <int,int> map_interior;
        std::map <int,int> map_exterior;

        MC_v3d_vector point_set_interior;
        MC_v3d_vector point_set_exterior;
        MC_connectivity_index connectivity_exterior;
        MC_connectivity_index connectivity_interior;

        MC_int_vector correspondance_interior;
        MC_int_vector correspondance_exterior;

        for(int k_poly=0;k_poly<N_poly;++k_poly)
        {
            MC_int_vector current_poly_index=connectivity()(k_poly);
            int N_current=current_poly_index.size();
            if(index_to_delete.find(k_poly)==index_to_delete_end) //do not delete
            {
                MC_int_vector exterior_polygon_index=MC_int_vector::zeros(N_current);
                for(int k_current=0;k_current<N_current;++k_current)
                {
                    std::pair<std::map<int,int>::iterator,bool> inserted_exterior=map_exterior.insert(std::make_pair(current_poly_index[k_current],map_exterior.size()));
                    if(inserted_exterior.second==true)//new index
                    {
                        exterior_polygon_index[k_current]=map_exterior.size()-1;
                        point_set_exterior.add(point_set()(current_poly_index[k_current]));
                        correspondance_exterior.add(current_poly_index[k_current]);
                    }
                    else //already existing vertex
                        exterior_polygon_index[k_current]=inserted_exterior.first->second;
                }
                connectivity_exterior.add(exterior_polygon_index);
            }
            else //index to delete
            {
                MC_int_vector interior_polygon_index=MC_int_vector::zeros(N_current);
                for(int k_current=0;k_current<N_current;++k_current)
                {
                    std::pair<std::map<int,int>::iterator,bool> inserted_interior=map_interior.insert(std::make_pair(current_poly_index[k_current],map_interior.size()));
                    if(inserted_interior.second==true)//new index
                    {
                        interior_polygon_index[k_current]=map_interior.size()-1;
                        point_set_interior.add(point_set()(current_poly_index[k_current]));
                        correspondance_interior.add(current_poly_index[k_current]);
                    }
                    else //already existing vertex
                        interior_polygon_index[k_current]=inserted_interior.first->second;
                }
                connectivity_interior.add(interior_polygon_index);
            }
        }

        return std::make_pair(std::make_pair(MC_mesh_index_vector(point_set_exterior,connectivity_exterior),MC_mesh_index_vector(point_set_interior,connectivity_interior)),std::make_pair(correspondance_exterior,correspondance_interior));
    }





    std::pair<std::pair<MC_mesh_index_vector,MC_mesh_index_vector>,std::pair<MC_int_vector,MC_int_vector> > MC_mesh_index_vector::delete_vertex(const std::set<int> index_to_delete) const
    {

        MC_mesh_index_vector mesh_deleted;
        std::map<int,int> map_vertex_deleted;
        std::map<int,int> :: iterator map_vertex_deleted_end=map_vertex_deleted.end();
        MC_int_vector correspondance_deleted;

        MC_mesh_index_vector mesh_undeleted;
        std::map<int,int> map_vertex_undeleted;
        std::map<int,int> :: iterator map_vertex_undeleted_end=map_vertex_undeleted.end();
        MC_int_vector correspondance_undeleted;

        int N_polygon=polygon_number();
        std::set<int> ::const_iterator it_index_to_delete_end=index_to_delete.end();
        for(int k_polygon=0;k_polygon<N_polygon;++k_polygon)
        {
            MC_int_vector current_polygon=connectivity()(k_polygon);
            int N_vertex=current_polygon.size();

            bool is_deleted_polygon=false;
            for(int k_vertex=0;k_vertex<N_vertex;++k_vertex)
                if(index_to_delete.find(current_polygon[k_vertex])!=it_index_to_delete_end)
                    is_deleted_polygon=true;

            //add the polygon in the deleted part
            if(is_deleted_polygon==true)
            {
                MC_int_vector temp_index;
                for(int k_vertex=0;k_vertex<N_vertex;++k_vertex)
                {
                    std::pair<std::map<int,int>::iterator,bool> it=map_vertex_deleted.insert(std::make_pair(current_polygon[k_vertex],map_vertex_deleted.size()));
                    if(it.second==true)
                    {
                        mesh_deleted.point_set().add(point_set()(current_polygon[k_vertex]));
                        correspondance_deleted.add(it.first->first);
                    }
                    temp_index.add(it.first->second);
                }
                mesh_deleted.connectivity().add(temp_index);
            }
            //add the polygon in the resulting part
            else
            {
                MC_int_vector temp_index;
                for(int k_vertex=0;k_vertex<N_vertex;++k_vertex)
                {

                    std::pair<std::map<int,int>::iterator,bool> it=map_vertex_undeleted.insert(std::make_pair(current_polygon[k_vertex],map_vertex_undeleted.size()));
                    if(it.second==true)
                    {
                        mesh_undeleted.point_set().add(point_set()(current_polygon[k_vertex]));
                        correspondance_undeleted.add(it.first->first);
                    }
                    temp_index.add(it.first->second);
                }
                mesh_undeleted.connectivity().add(temp_index);
            }
        }


        return std::make_pair(std::make_pair(mesh_undeleted,mesh_deleted),std::make_pair(correspondance_undeleted,correspondance_deleted));

    }

    std::pair<std::pair<MC_mesh_index_vector,MC_mesh_index_vector>,std::pair<MC_int_vector,MC_int_vector> > MC_mesh_index_vector::delete_boundary_polygon() const
    {
        std::set<int> border_polygon=connectivity_mesh.boundary_polygon();
        return delete_polygon(border_polygon);
    }



    std::vector<MC_polygon> MC_mesh_index_vector::get_polygon() const
    {

        int N=polygon_number();
        std::vector<MC_polygon> v_poly(N);
        for(int k=0;k<N;++k)
            v_poly[k]=get_polygon(k);
        return v_poly;
    }

    MC_mesh_index_vector MC_mesh_index_vector::subdivide_mid_edge_unchanged_boundary() const
    {
        std::set<MC_int_pair,MC_int_pair_less> s;
        return subdivide_mid_edge_unchanged_boundary(s);
    }
    MC_mesh_index_vector MC_mesh_index_vector::subdivide_barycenter_mid_edge_unchanged_boundary() const
    {
        std::set<MC_int_pair,MC_int_pair_less> s;
        return subdivide_barycenter_mid_edge_unchanged_boundary(s);
    }
    MC_mesh_index_vector MC_mesh_index_vector::subdivide_mixed_mid_edge_unchanged_boundary() const
    {
        std::set<MC_int_pair,MC_int_pair_less> s;
        return subdivide_mixed_mid_edge_unchanged_boundary(s);
    }

    MC_mesh_index_vector MC_mesh_index_vector::subdivide_mixed_mid_edge(const std::set<int>& polygon_to_subdivide) const
    {
        std::set<MC_int_pair,MC_int_pair_less> s;
        return subdivide_mixed_mid_edge(polygon_to_subdivide,s);
    }
    MC_mesh_index_vector MC_mesh_index_vector::subdivide_mixed_mid_edge(const std::set<int>& polygon_to_subdivide,const std::set<MC_int_pair,MC_int_pair_less>& allowed_edge) const
    {

        MC_v3d_vector new_mesh_vector=point_set();
        MC_connectivity_index new_mesh_connectivity=connectivity().delete_index(MC_int_vector(polygon_to_subdivide)).first;
        MC_mesh_index_vector new_mesh(new_mesh_vector,new_mesh_connectivity);

        std::pair <std::pair<MC_mesh_index_vector,MC_mesh_index_vector>,std::pair<MC_int_vector,MC_int_vector> > splitted=delete_polygon(polygon_to_subdivide);

        std::map <int,int> new_to_original=splitted.second.second.to_map();
        std::map <int,int> original_to_new=MC_helper_stl::reverse_map(new_to_original);

        std::set<MC_int_pair,MC_int_pair_less> new_allowed_edge;
        for(std::set<MC_int_pair,MC_int_pair_less>::const_iterator
            it=allowed_edge.begin(),
            it_end=allowed_edge.end();
        it!=it_end;++it)
            new_allowed_edge.insert(MC_int_pair(new_to_original[(*it)[0]],new_to_original[(*it)[1]]));

        MC_mesh_index_vector subdivided=splitted.first.second.subdivide_mixed_mid_edge_unchanged_boundary(new_allowed_edge);
        int N_vertex_old=splitted.first.second.point_set().size();
        int N_vertex_new=subdivided.point_set().size();



        for(int k=N_vertex_old;k<N_vertex_new;++k)
            new_mesh.point_set().add(subdivided.point_set()(k));
        int N_subdivided_poly=subdivided.polygon_number();
        for(int k=0;k<N_subdivided_poly;++k)
        {
            MC_int_vector new_poly=subdivided.connectivity()(k);
            int N_new_poly=new_poly.size();
            MC_int_vector new_poly_in_full_coordinate=MC_int_vector::zeros(N_new_poly);
            for(int k_index=0;k_index<N_new_poly;++k_index)
            {
                int u=new_poly[k_index];
                if(u<N_vertex_old)
                    new_poly_in_full_coordinate[k_index]=splitted.second.second(u);
                else
                    new_poly_in_full_coordinate[k_index]=(u-N_vertex_old)+point_set().size();
            }
            new_mesh.connectivity().add(new_poly_in_full_coordinate);
        }

        new_mesh.connectivity().build_ring().build_star();

        return new_mesh;
    }
    std::pair <MC_int_vector,MC_int_vector> MC_mesh_index_vector::polygons_inside_sphere(const MC_v3d& center,const double& radius) const
    {
        MC_int_vector index_inside,index_outside;
        for(int k_polygon=0,N_polygon=polygon_number();k_polygon<N_polygon;++k_polygon)
        {
            bool is_inside=false;
            MC_int_vector poly=connectivity()(k_polygon);
            for(int k=0,N=poly.size();is_inside==false&&k<N;++k)
            {
                if( (point_set()(poly[k])-center).norm()<=radius)
                {
                    is_inside=true;
                    index_inside.add(k_polygon);

                }
            }
            if(is_inside==false)
                index_outside.add(k_polygon);
        }
        return std::make_pair(index_inside,index_outside);
    }

    MC_mesh_index_vector MC_mesh_index_vector::laplacian_smoothing(const double& lambda,const int& steps,const bool& is_boundary_preserving) const
    {

        if(steps==0)
            return *this;

        MC_mesh_index_vector new_mesh=*this;

        //ring need to be computed
        std::map<int,std::set<int> > r=connectivity().ring();

        std::set<int> bnd=connectivity().boundary_vertex();


        for(int k_step=0;k_step<steps;++k_step)
        {
            MC_v3d_vector temp=MC_v3d_vector::zeros(vertex_number());
            for(int k_vertex=0,N_vertex=vertex_number();k_vertex<N_vertex;++k_vertex)
            {
                MC_v3d x=new_mesh.point_set()(k_vertex);

                if(is_boundary_preserving==false ||
                   (is_boundary_preserving==true && bnd.find(k_vertex)==bnd.end()) )
                {//not a boundary vertex

                    MC_v3d bar;
                    std::set<int> current_ring=r[k_vertex];
                    for(std::set<int>::const_iterator it=current_ring.begin(),it_end=current_ring.end();it!=it_end;++it)
                        bar += new_mesh.point_set()(*it);

                    if(current_ring.size()>0)
                        bar/=current_ring.size();

                    temp[k_vertex]=(1-lambda)*x+lambda*bar;
                }
                else//boundary
                    temp[k_vertex]=x;
            }
            new_mesh.point_set()=temp;
        }

        return new_mesh;
    }

    MC_mesh_index_vector MC_mesh_index_vector::load_mesh_file(const std::string& filename)
    {
        MC_mesh_index_vector mesh;

        //get extension
        std::vector<std::string> token=MC_string_tokenizer::tokenize(filename,".");
        std::string extension=token[token.size()-1];

        if(extension.find("off")!=std::string::npos)
            mesh=MC_io_off::read_off_file(filename);
        else if(extension.find("obj")!=std::string::npos)
            mesh=MC_io_obj::read_obj_file(filename);
        else
            std::cout<<"Warning in MC_mesh_index_vector::load_mesh_file("<<filename<<"), extension "<<extension<<" not recognized"<<std::endl;

        return mesh;
    }

    double MC_mesh_index_vector::area() const
    {
        double a=0.0;
        for(int k=0,N=polygon_number();k<N;++k)
        {
            MC_int_vector poly=connectivity_mesh(k);

            MC_v3d x0=point_set_mesh(poly[0]);
            for(int k_tri=0,N_pol=poly.size();k_tri<N_pol-2;++k_tri)
            {
                MC_v3d x1=point_set_mesh(poly[k_tri+1]);
                MC_v3d x2=point_set_mesh(poly[k_tri+2]);

                double C0=(x2[1]-x0[1])*(x1[2]-x0[2])-(x2[2]-x0[2])*(x1[1]-x0[1]);
                double C1=(x2[2]-x0[2])*(x1[0]-x0[0])-(x2[0]-x0[0])*(x1[2]-x0[2]);
                double C2=(x2[0]-x0[0])*(x1[1]-x0[1])-(x2[1]-x0[1])*(x1[0]-x0[0]);

                double u=sqrt(C0*C0+C1*C1+C2*C2);

                a+=0.5*u;
            }
        }
        return a;
    }

    MC_v3d_vector MC_mesh_index_vector::area_gradient() const
    {
        MC_v3d_vector grad=MC_v3d_vector::zeros(vertex_number());

        for(int k=0,N=polygon_number();k<N;++k)
        {
            MC_int_vector poly=connectivity_mesh(k);

            MC_v3d x0=point_set_mesh(poly[0]);
            int index_0=poly[0];
            for(int k_tri=0,N_pol=poly.size();k_tri<N_pol-2;++k_tri)
            {
                MC_v3d x1=point_set_mesh(poly[k_tri+1]);
                MC_v3d x2=point_set_mesh(poly[k_tri+2]);


                int index_1=poly[k_tri+1];
                int index_2=poly[k_tri+2];

                double C0=(x2[1]-x0[1])*(x1[2]-x0[2])-(x2[2]-x0[2])*(x1[1]-x0[1]);
                double C1=(x2[2]-x0[2])*(x1[0]-x0[0])-(x2[0]-x0[0])*(x1[2]-x0[2]);
                double C2=(x2[0]-x0[0])*(x1[1]-x0[1])-(x2[1]-x0[1])*(x1[0]-x0[0]);

                double u=sqrt(C0*C0+C1*C1+C2*C2);


                grad[index_0][0] +=  ( (x1[2]-x2[2])*C1 + (x2[1]-x1[1])*C2 )/(2*u);
                grad[index_0][1] +=  ( (x2[2]-x1[2])*C0 + (x1[0]-x2[0])*C2 )/(2*u);
                grad[index_0][2] +=  ( (x1[1]-x2[1])*C0 + (x2[0]-x1[0])*C1 )/(2*u);

                grad[index_1][0] +=  ( (x2[2]-x0[2])*C1 + (x0[1]-x2[1])*C2 )/(2*u);
                grad[index_1][1] +=  ( (x0[2]-x2[2])*C0 + (x2[0]-x0[0])*C2 )/(2*u);
                grad[index_1][2] +=  ( (x2[1]-x0[1])*C0 + (x0[0]-x2[0])*C1 )/(2*u);

                grad[index_2][0] +=  ( (x0[2]-x1[2])*C1 + (x1[1]-x0[1])*C2 )/(2*u);
                grad[index_2][1] +=  ( (x1[2]-x0[2])*C0 + (x0[0]-x1[0])*C2 )/(2*u);
                grad[index_2][2] +=  ( (x0[1]-x1[1])*C0 + (x1[0]-x0[0])*C1 )/(2*u);
            }
        }

        return grad;
    }


    MC_mesh_index_vector MC_mesh_index_vector::build_ball_point_set(const MC_v3d_vector& v,const double& radius,const int& N_subdiv)
    {
        MC_mesh_index_vector sph=MC_mesh_index_vector::build_sphere(N_subdiv)*radius;
        MC_mesh_index_vector visu_point_set;
        for(int k=0,N=v.size();k<N;++k)
            visu_point_set.concatenation(sph+v[k]);
        return visu_point_set;
    }

    double MC_mesh_index_vector::average_edge_length() const
    {
        int counter=0;
        double avg_length=0.0;
        for(int k=0,N=polygon_number();k<N;++k)
        {
            MC_int_vector index=connectivity()(k);
            for(int k_v=0,N_v=index.size();k_v<N_v;++k_v)
            {
                avg_length += ( point_set()(index(k_v))-point_set()(index( (k_v+1)%N_v )) ).norm();
                ++counter;
            }
        }
        return avg_length/static_cast<double>(counter);
    }

    MC_mesh_index_vector MC_mesh_index_vector::get_polygon_mesh() const
    {
        MC_mesh_index_vector exploded;
        for(int k=0,N=polygon_number();k<N;++k)
        {
            MC_polygon p=get_polygon(k);
            exploded.connectivity().add(MC_int_vector::linspace(0,p.size()-1)+exploded.vertex_number());
            exploded.point_set().add(p);
        }
        return exploded;
    }




    MC_mesh_index_vector MC_mesh_index_vector::build_wireframe(const MC_mesh_index_vector& mesh,const double& radius_cylinder,const int& N_circular,const int& N_subdiv_edges)
    {
        std::set<MC_int_pair,MC_int_pair_less> already_created_edge;

        MC_curve pattern=MC_curve::build_circle(radius_cylinder,N_circular);
        MC_mesh_index_vector wireframe;

        for(int k=0,N=mesh.polygon_number();k<N;++k)
        {
            MC_int_vector poly=mesh.connectivity()(k);
            for(int k2=0,N_poly=poly.size();k2<N_poly;++k2)
            {
                int index_0=poly[k2];
                int index_1=poly[(k2+1)%N_poly];

                if(already_created_edge.insert(MC_int_pair(index_0,index_1)).second==true)
                {
                    MC_v3d x0=mesh.point_set()(poly[k2]);
                    MC_v3d x1=mesh.point_set()(poly[(k2+1)%N_poly]);

                    MC_curve c=MC_curve::build_line(x0,x1,N_subdiv_edges);

                    MC_mesh_index_vector sweep=MC_mesh_index_vector::sweep_surface(c,pattern,2,false);
                    wireframe.concatenation(sweep);
                }
            }
        }

        return wireframe;
    }

    MC_matrix MC_mesh_index_vector::inertia() const
    {
        MC_matrix J(3);
        for(int k=0,N=polygon_number();k<N;++k)
        {
            std::vector<MC_triangle> tri=MC_triangle::triangulate(get_polygon(k));
            for(int k_tri=0,N_tri=tri.size();k_tri<N_tri;++k_tri)
                J+=tri[k_tri].inertia();
        }

        return J;
    }

    MC_mesh_index_vector MC_mesh_index_vector::build_sphere_point_set(const MC_v3d_vector& position,const double& radius,const unsigned int& N_subdiv)
    {
        MC_mesh_index_vector sph=radius*MC_mesh_index_vector::build_sphere(N_subdiv);
        MC_mesh_index_vector v_sph;
        for(int k=0,N=position.size();k<N;++k)
            v_sph.concatenation(sph+position[k]);

        return v_sph;
    }



    MC_curve MC_mesh_index_vector::intersection_curve(const MC_mesh_index_vector& mesh,const MC_segment& s)
    {
        //get the normal plane
        MC_v3d_vector normal_mesh=mesh.get_polygon(0).normal();
        MC_v3d_vector normal_plane=(s[1]-s[0]).normalized().cross(normal_mesh).normalized();


        //get the set of curve by intersecting the mesh with a plane
        std::vector <MC_curve> curve_inter_plane=mesh.plane_intersection(normal_plane,s[0]);



        MC_io_off::write_off_file("mesh_to_cut.off",mesh);
        std::vector<MC_segment> v_s;v_s.push_back(s);
        MC_io_off::write_off_file("line_in_mesh.off",MC_mesh_index_vector::build_segment(v_s,0.005,10));

         std::cout<<" ! "<<curve_inter_plane.size()<<std::endl;

        std::cout<<"-- "<<curve_inter_plane[0]<<std::endl;;
        //std::cout<<normal_mesh<<" "<<s<<curve_inter_plane.size()<<std::endl;
        if(curve_inter_plane.size()<1)
        {std::cout<<"Something strange in MC_mesh_index_vector::intersection_curve()"<<std::endl;exit(-1);}

        return curve_inter_plane[0];

    }

    MC_mesh_index_vector MC_mesh_index_vector::build_segment(const std::vector<MC_segment>& v_seg,const double& radius,const double& N_circular)
    {
        MC_curve pattern=MC_curve::build_circle(radius,N_circular);
        MC_mesh_index_vector wireframe;
        unsigned int N_subdiv_edges=2;
        for(int k=0,N=v_seg.size();k<N;++k)
        {
            const MC_v3d& x0=v_seg[k][0];
            const MC_v3d& x1=v_seg[k][1];

            MC_curve c=MC_curve::build_line(x0,x1,N_subdiv_edges);
            MC_mesh_index_vector sweep=MC_mesh_index_vector::sweep_surface(c,pattern,8,false);
            wireframe.concatenation(sweep);
        }
        return wireframe;
    }

    MC_mesh_index_vector MC_mesh_index_vector::texture_convert_planar_xy(const MC_mesh_index_vector& mesh)
    {

        MC_v3d_vector texture_coord=mesh.point_set();
        texture_coord=texture_coord.scaled_to_unit_and_center().first;//+MC_v3d(0.5,0.5,0.0);
        return MC_mesh_index_vector(texture_coord,mesh.connectivity());
    }
    MC_mesh_index_vector MC_mesh_index_vector::texture_convert_planar_xy_same_connectivity(const MC_mesh_index_vector& mesh_tex,const MC_mesh_index_vector& mesh_ref)
    {
        MC_v3d_vector new_texture;
        MC_mesh_index_vector tex=texture_convert_planar_xy(mesh_tex);
        for(unsigned int k_poly=0,N_poly=mesh_ref.polygon_number();k_poly<N_poly;++k_poly)
            for(unsigned int k=0,N=mesh_ref.connectivity()[k_poly].size();k<N;++k)
                new_texture.set( mesh_ref.connectivity()[k_poly][k],tex.point_set()(tex.connectivity()[k_poly][k]) );
        tex=MC_mesh_index_vector(new_texture,mesh_ref.connectivity());
        return tex;
    }

    std::pair<MC_mesh_index_vector,std::pair<MC_int_vector_vector,MC_int_vector_vector> > MC_mesh_index_vector::build_from_polygon_soup(const std::vector<MC_polygon> poly_soup)
    {
        MC_mesh_index_vector m;
        return m.added_polygon_soup(poly_soup);
    }

    MC_mesh_index_vector MC_mesh_index_vector::build_square(const int& N1,const int& N2,const MC_v3d& tangent_1,const MC_v3d& tangent_2,const MC_v3d& center,const double& L1,const double& L2)
    {
        MC_mesh_index_vector m=build_square(N1,N2);

        //center
        m.point_set() += MC_v3d(-0.5,-0.5,0.0);

        //scale
        m.point_set()=m.point_set().scale(MC_v3d(L1,L2,1.0));



        //rotate
        MC_matrix R1=MC_matrix::rotation_axis_to_axis(MC_v3d(1,0,0),tangent_1);
        MC_matrix R2=MC_matrix::rotation_axis_to_axis(R1*MC_v3d(0,1,0),tangent_2);
        m=R2*R1*m;

        //translate
        m.point_set() += center;

        return m;
    }

}
